/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004 by Michael Buesch                            *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef CARDIF_H
#define CARDIF_H

#include <qthread.h>
#include <qobject.h>

#include <stdint.h>
#include <string>
using std::string;

/** open() timeout in seconds */
#define CARDIF_OPEN_TIMEOUT	30

/** CardIf status notification signals */
enum cardif_status {
	status_success = 0,
	status_failure,
	status_openErr,
	status_noCard,
	status_invalidCard,
	status_invalidVersion,
	status_tooSmall,
	status_writeErr
};

enum cardif_task {
	task_open = 0,
	task_close,
	task_genNewCard,
	task_writeNewCard,
	task_getKey,
	task_eraseCard
};

class ThreadComm;

class CardIf_templ : public QThread
		   , public QObject
{
public:
	CardIf_templ(unsigned int thread_stackSize)
	 : QThread(thread_stackSize)
	 , QObject()
	{}
	~CardIf_templ()
	{}
};

/** chipcard low-level interface class */
class CardIf : public CardIf_templ
{
	Q_OBJECT
public:
	CardIf(unsigned int thread_stackSize = 0);
	~CardIf();

	/** open card */
	bool open();
	/** close card */
	void close();
	/** generate a new card */
	void genNewCard();
	/** write new card-data */
	void writeNewCard(uint32_t cardId, const string &key);
	/** get key of inserted card */
	void getKey();
	/** erase the inserted card */
	void eraseCard();
	/** cancel all currently running operations */
	void cancelOp();

	/** generate a new random key */
	static string genKey(unsigned long size);
	/** generate a new random card-id */
	static uint32_t genCardId();

signals:
	/** open() status signal */
	void open_status(cardif_status stat);
	/** close() status signal */
	void close_status(cardif_status stat);
	/** genNewCard() status signal */
	void genNewCard_status(cardif_status stat);
	/** writeNewCard() status signal */
	void writeNewCard_status(cardif_status stat);
	/** getKey() status signal */
	void getKey_status(cardif_status stat, uint32_t cardId,
			   const string &key);
	/** eraseCard() status signal */
	void eraseCard_status(cardif_status stat);

protected:
	/** main thread-process */
	void run();
	/* run() task functions */
	void run_open();
	void run_close();
	void run_close_nosig();
	void run_genNewCard();
	void run_writeNewCard();
	int run_writeCard(uint32_t cardId, const string &key);
	void run_getKey();
	void run_eraseCard();

protected slots:
	/** interprocess communication data slot */
	void ipc_data(const string *line);

protected:
	/** interprocess communication host */
	ThreadComm *threadComm_host;
};

#endif
