/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004 by Michael Buesch                                  *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "configuration.h"
#if KDE_VERSION >= KDE_MAKE_VERSION(3, 2, 0)

#include <kconfigdialog.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <kfontrequester.h>

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qfileinfo.h>
#include <qfontdialog.h>
#include <qspinbox.h>
#include <qvalidator.h>
#include <qsqlpropertymap.h>

#include <stdlib.h>
#include <stdio.h>

#define INITIAL_CONFWND_SIZE	(QSize(600, 450))


/*********************************************************
 **  class Configuration                                **
 *********************************************************/

Configuration * Configuration::_obj (0);

Configuration::Configuration()
{
	skel = new KConfigSkeleton;
	initSkel();
	readConfig();
}

Configuration::~Configuration()
{
	writeConfig();
	delete_ifnot_null(skel);
}

void Configuration::initSkel()
{
	skel->setCurrentGroup("GLOBAL");
	skel->addItemString("autoStart", cGlobAutoStart);
	skel->addItemString("bgPix", cGlobBgPix, CONF_DEFAULT_BGPATH);
	skel->addItemString("browserCommand", cGlobBrowserCommand, CONF_DEFAULT_BROWSERCOMMAND);
	skel->addItemString("xtermCommand", cGlobXtermCommand, CONF_DEFAULT_XTERMCOMMAND);
	skel->addItemFont("entryFont", cGlobEntryFont);
	skel->addItemInt("pwTimeout", cGlobPwTimeout, CONF_DEFAULT_PWTIMEOUT);
	skel->addItemInt("lockTimeout", cGlobLockTimeout, CONF_DEFAULT_LOCKTIMEOUT);
	skel->addItemInt("compression", cGlobCompression, CONF_DEFAULT_COMPRESSION);
	skel->addItemInt("filePermissions", cGlobFilePermissions, CONF_DEFAULT_FILEPERMISSIONS);
	skel->addItemInt("minimizeLock", cGlobMinimizeLock, CONF_DEFAULT_MINIMIZELOCK);
	skel->addItemInt("pwGen", cGlobPwGen, CONF_DEFAULT_PWGEN);
	skel->addItemBool("unlockOnOpen", cGlobUnlockOnOpen, CONF_DEFAULT_UNLOCKONOPEN);
	skel->addItemBool("tray", cGlobTray, CONF_DEFAULT_TRAY);
	skel->addItemBool("makeFileBackup", cGlobMakeFileBackup, CONF_DEFAULT_MAKEFILEBACKUP);
	skel->addItemBool("autostartDeeplocked", cGlobAutostartDeepLocked, CONF_DEFAULT_AUTOSTART_DEEPL);
	skel->addItemBool("autoDeepLock", cGlobAutoDeepLock, CONF_DEFAULT_AUTODEEPLOCK);
	skel->addItemBool("kwalletEmu", cGlobKwalletEmu, CONF_DEFAULT_KWALLETEMU);
	skel->addItemBool("enableDcop", cGlobEnableDcop, CONF_DEFAULT_ENABLEDCOP);

	skel->setCurrentGroup("WND");
	skel->addItemSize("MainWndSize", cWndMainWndSize);
	skel->addItemInt("MainViewStyle", cWndMainViewStyle, CONF_DEFAULT_MAINVIEWSTYLE);
	skel->addItemBool("autoMinimizeOnStart", cWndAutoMinimizeOnStart, CONF_DEFAULT_AUTOMINIMIZE);
}

bool Configuration::showConfWnd(QWidget *parent)
{
	bool ret = true;
	KConfigDialog *confDlg;
	confDlg = new KConfigDialog(parent, i18n("Main configuration"), skel,
				    KDialogBase::IconList,
				    KConfigDialog::Default | KConfigDialog::Ok |
				    KConfigDialog::Cancel | KConfigDialog::Help,
				    KConfigDialog::Ok, true);
	ConfPageGlobal *confPageGlobal = new ConfPageGlobal;
	ConfPageLookNFeel *confPageLookNFeel = new ConfPageLookNFeel;
	ConfPageFile *confPageFile = new ConfPageFile;
	ConfPageTimeouts *confPageTimeouts = new ConfPageTimeouts;
	ConfPageExtApps *confPageExtApps = new ConfPageExtApps;
	ConfPageAutostart *confPageAutostart = new ConfPageAutostart;
	confDlg->addPage(confPageGlobal, i18n("General"), "pwmanager");
	confDlg->addPage(confPageLookNFeel, i18n("Look & Feel"), "fonts");
	confDlg->addPage(confPageFile, i18n("Files"), "filesave");
	confDlg->addPage(confPageTimeouts, i18n("Timeouts"), "clock");
	confDlg->addPage(confPageExtApps, i18n("External Applications"), "gear");
	confDlg->addPage(confPageAutostart, i18n("Autostart"), "fileopen");
	confDlg->resize(INITIAL_CONFWND_SIZE);
	if (confDlg->exec() == 0)
		ret = false;
	delete confPageGlobal;
	delete confPageLookNFeel;
	delete confPageFile;
	delete confPageTimeouts;
	delete confPageExtApps;
	delete confPageAutostart;
	return ret;
}


/*********************************************************
 **  class OctLineEdit                                  **
 *********************************************************/

OctLineEdit::OctLineEdit(QWidget *parent, const char *name)
 : QLineEdit(parent, name)
{
}

OctLineEdit::~OctLineEdit()
{
}

void OctLineEdit::setText(const QString &t)
{
	bool ok;
	int tmp = t.toInt(&ok, 10);
	if (!ok)
		return;
	QString ret;
	ret.setNum(tmp, 8);
	QLineEdit::setText(ret);
}

QString OctLineEdit::text() const
{
	bool ok;
	QString ret;
	QString t(QLineEdit::text());
	int tmp = t.toInt(&ok, 8);
	if (!ok)
		return ret;
	ret.setNum(tmp, 10);
	return ret;
}

void OctLineEdit::keyPressEvent(QKeyEvent *e)
{
	int key = e->key();
	switch (key) {
	case Qt::Key_0:
	case Qt::Key_1:
	case Qt::Key_2:
	case Qt::Key_3:
	case Qt::Key_4:
	case Qt::Key_5:
	case Qt::Key_6:
	case Qt::Key_7:
	case Qt::Key_Escape:
	case Qt::Key_Backtab:
	case Qt::Key_Backspace:
	case Qt::Key_Return:
	case Qt::Key_Enter:
	case Qt::Key_Insert:
	case Qt::Key_Delete:
	case Qt::Key_Home:
	case Qt::Key_End:
	case Qt::Key_Left:
	case Qt::Key_Up:
	case Qt::Key_Right:
	case Qt::Key_Down:
		e->accept();
		QLineEdit::keyPressEvent(e);
		break;
	default:
		e->ignore();
	}
}


/*********************************************************
 **  class ConfPageGlobal                               **
 *********************************************************/

ConfPageGlobal::ConfPageGlobal(QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
	QHBox *hbox1;
	QCheckBox *kcfg_tray;
	QCheckBox *kcfg_autoMinimizeOnStart;
	QCheckBox *kcfg_unlockOnOpen;
	QCheckBox *kcfg_enableDcop;
	QCheckBox *kcfg_kwalletEmu;
	QComboBox *kcfg_minimizeLock;
	QLabel *kcfg_minimizeLock_label;

	vbox1 = new QVBox(this);
	kcfg_tray = new QCheckBox(i18n("Show icon in system-tray"), vbox1, "kcfg_tray");
	kcfg_autoMinimizeOnStart = new QCheckBox(i18n("auto-minimize to tray on startup"),
						 vbox1, "kcfg_autoMinimizeOnStart");
	kcfg_unlockOnOpen = new QCheckBox(i18n("Open document with passwords unlocked"),
					  vbox1, "kcfg_unlockOnOpen");
	kcfg_enableDcop = new QCheckBox(i18n("enable DCOP interface"),
					vbox1, "kcfg_enableDcop");
	kcfg_kwalletEmu = new QCheckBox(i18n("KWallet emulation"),
					vbox1, "kcfg_kwalletEmu");
	hbox1 = new QHBox(vbox1);
	kcfg_minimizeLock_label = new QLabel(i18n("auto-lock on minimize:"), hbox1);
	kcfg_minimizeLock = new QComboBox(hbox1, "kcfg_minimizeLock");
	kcfg_minimizeLock->insertItem(i18n("don't lock"));
	kcfg_minimizeLock->insertItem(i18n("normal lock"));
	kcfg_minimizeLock->insertItem(i18n("deep-lock"));
}


/*********************************************************
 **  class ConfPageLookNFeel                            **
 *********************************************************/

ConfPageLookNFeel::ConfPageLookNFeel(QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
	QHBox *hbox1;
	QGroupBox *kcfg_bgPix_box;
	QPushButton *kcfg_bgPix_button;
	QComboBox *kcfg_MainViewStyle;
	QLabel *kcfg_MainViewStyle_label;
	KFontRequester *kcfg_entryFont;
	QGroupBox *kcfg_entryFont_box;

	// bg-path
	vbox1 = new QVBox(this);
	kcfg_bgPix_box = new QGroupBox(i18n("Path to background picture for main-window:"),
				       vbox1);
	kcfg_bgPix_box->setColumns(2);
	kcfg_bgPix = new QLineEdit(kcfg_bgPix_box, "kcfg_bgPix");
	kcfg_bgPix_button = new QPushButton("...", kcfg_bgPix_box);
	// font
	kcfg_entryFont_box = new QGroupBox(i18n("Font for the password entries:"), vbox1);
	kcfg_entryFont_box->setColumns(1);
	kcfg_entryFont = new KFontRequester(kcfg_entryFont_box, "kcfg_entryFont", false);
	// wnd style
	hbox1 = new QHBox(vbox1);
	kcfg_MainViewStyle_label = new QLabel(i18n("Window-style:"), hbox1);
	kcfg_MainViewStyle = new QComboBox(hbox1, "kcfg_MainViewStyle");
	kcfg_MainViewStyle->insertItem(i18n("Category on top"));
	kcfg_MainViewStyle->insertItem(i18n("Category-list left"));
	// connections
	connect(kcfg_bgPix_button, SIGNAL(clicked()),
		this, SLOT(browseButton_slot()));
}

void ConfPageLookNFeel::browseButton_slot()
{
	QFileInfo fi(CONF_DEFAULT_BGPATH);
	QString initPath(fi.isReadable() ? fi.dirPath(TRUE) : QDir::home().
			 absPath());
	QString path = KFileDialog::getImageOpenURL(initPath).path();
	if (path != QString::null)
		kcfg_bgPix->setText(path);
}


/*********************************************************
 **  class ConfPageFile                                 **
 *********************************************************/

ConfPageFile::ConfPageFile(QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
	QComboBox *kcfg_compression;
	QLabel *kcfg_compression_label;
	OctLineEdit *kcfg_filePermissions;
	QLabel *kcfg_filePermissions_label;
	QCheckBox *kcfg_makeFileBackup;
	QHBox *hbox1;
	QHBox *hbox2;

	vbox1 = new QVBox(this);
	// compression
	hbox1 = new QHBox(vbox1);
	kcfg_compression_label = new QLabel(i18n("*.pwm file compression:"), hbox1);
	kcfg_compression = new QComboBox(hbox1, "kcfg_compression");
	kcfg_compression->insertItem(i18n("none"));
	kcfg_compression->insertItem(i18n("gzip"));
	kcfg_compression->insertItem(i18n("bzip2"));
	// permissions
	hbox2 = new QHBox(vbox1);
	kcfg_filePermissions_label = new QLabel(i18n("permissions:"), hbox2);
	kcfg_filePermissions = new OctLineEdit(hbox2, "kcfg_filePermissions");
	kcfg_filePermissions->setMaxLength(3);
	// backup
	kcfg_makeFileBackup = new QCheckBox(i18n("Make file backup before saving"),
					    vbox1, "kcfg_makeFileBackup");
}


/*********************************************************
 **  class ConfPageTimeouts                             **
 *********************************************************/

ConfPageTimeouts::ConfPageTimeouts(QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
	QSpinBox *kcfg_pwTimeout;
	QLabel *kcfg_pwTimeout_label;
	QSpinBox *kcfg_lockTimeout;
	QLabel *kcfg_lockTimeout_label;
	QCheckBox *kcfg_autoDeepLock;
	QHBox *hbox1;
	QHBox *hbox2;

	vbox1 = new QVBox(this);
	// pw timeout
	hbox1 = new QHBox(vbox1);
	kcfg_pwTimeout_label = new QLabel(i18n("Password timeout (timeout to hold "
					       "password in memory, so you don't have "
					       "to re-enter it, if you already have "
					       "entered it) [set to 0 to disable]:"),
					  hbox1);
	kcfg_pwTimeout_label->setAlignment(QLabel::WordBreak);
	kcfg_pwTimeout = new QSpinBox(hbox1, "kcfg_pwTimeout");
	// lock timeout
	hbox2 = new QHBox(vbox1);
	kcfg_lockTimeout_label = new QLabel(i18n("Auto-lock timeout (auto lock document "
						 "after this amount of seconds) "
						 "[set to 0 to disable]:"),
					    hbox2);
	kcfg_lockTimeout_label->setAlignment(QLabel::WordBreak);
	kcfg_lockTimeout = new QSpinBox(hbox2, "kcfg_lockTimeout");
	// auto deep lock checkbox
	kcfg_autoDeepLock = new QCheckBox(i18n("deep-lock on autolock"),
					  vbox1, "kcfg_autoDeepLock");
}


/*********************************************************
 **  class ConfPageExtApps                              **
 *********************************************************/

ConfPageExtApps::ConfPageExtApps(QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
	QLineEdit *kcfg_browserCommand;
	QLabel *kcfg_browserCommand_label;
	QLineEdit *kcfg_xtermCommand;
	QLabel *kcfg_xtermCommand_label;
	QComboBox *kcfg_pwGen;
	QLabel *kcfg_pwGen_label;
	QHBox *hbox1;
	QHBox *hbox2;
	QHBox *hbox3;

	vbox1 = new QVBox(this);
	// browser command
	hbox1 = new QHBox(vbox1);
	kcfg_browserCommand_label = new QLabel(i18n("Favourite browser:"), hbox1);
	kcfg_browserCommand = new QLineEdit(hbox1, "kcfg_browserCommand");
	// xterm command
	hbox2 = new QHBox(vbox1);
	kcfg_xtermCommand_label = new QLabel(i18n("Favourite X-terminal:"), hbox2);
	kcfg_xtermCommand = new QLineEdit(hbox2, "kcfg_xtermCommand");
	// pw generator
	hbox3 = new QHBox(vbox1);
	kcfg_pwGen_label = new QLabel(i18n("password generator:"), hbox3);
	kcfg_pwGen = new QComboBox(hbox3, "kcfg_pwGen");
	kcfg_pwGen->insertItem("makepasswd");
	kcfg_pwGen->insertItem("PwG");
}


/*********************************************************
 **  class ConfPageAutostart                            **
 *********************************************************/

ConfPageAutostart::ConfPageAutostart(QWidget *parent, const char *name, WFlags f)
 : QWidget(parent, name, f)
{
	QGroupBox *kcfg_autoStart_box;
	QPushButton *kcfg_autoStart_button;
	QCheckBox *kcfg_autostartDeeplocked;

	vbox1 = new QVBox(this);
	// autostart
	kcfg_autoStart_box = new QGroupBox(i18n("Open this file automatically on startup:"),
					   vbox1);
	kcfg_autoStart_box->setColumns(2);
	kcfg_autoStart = new QLineEdit(kcfg_autoStart_box, "kcfg_autoStart");
	kcfg_autoStart_button = new QPushButton("...", kcfg_autoStart_box);
	kcfg_autostartDeeplocked = new QCheckBox(i18n("open deeplocked"),
						 kcfg_autoStart_box, "kcfg_autostartDeeplocked");
	// connections
	connect(kcfg_autoStart_button, SIGNAL(clicked()),
		this, SLOT(browseButton_slot()));
}

void ConfPageAutostart::browseButton_slot()
{
	QString path(KFileDialog::getOpenFileName(QString::null,
						  i18n("*.pwm|PwM Password file\n"
						       "*|All files"), this));
	if (path == QString::null)
		return;
	kcfg_autoStart->setText(path);
}

#include "configuration.moc"

#else // KDE_VERSION >= KDE_MAKE_VERSION(3, 2, 0)
  /* XXX: This is the code for KDE-3.1 compatibility. */
# include "configuration_31compat.cpp"
#endif // KDE_VERSION >= KDE_MAKE_VERSION(3, 2, 0)
