/*************************************************************************
 *                                                                       *
 *   copyright (C) 2003, 2004 by Michael Buesch                          *
 *   email: mbuesch@freenet.de                                           *
 *                                                                       *
 *  This library is free software; you can redistribute it and/or        *
 *  modify it under the terms of the GNU Lesser General Public           *
 *  License version 2.1 as published by the Free Software Foundation.    *
 *                                                                       *
 *  This library is distributed in the hope that it will be useful,      *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    *
 *  Lesser General Public License for more details.                      *
 *                                                                       *
 *  You should have received a copy of the GNU Lesser General Public     *
 *  License along with this library; if not, write to the Free Software  *
 *  Foundation, Inc., 59 Temple Place,                                   *
 *  Suite 330, Boston, MA 02111-1307  USA                                *
 *                                                                       *
 *************************************************************************/

#include "pwmanagerdcop.h"
#include "pwmanagerdcopif_stub.h"


PwManagerDCOP::PwManagerDCOP(QObject *parent, const char *name)
 : QObject(parent, name)
{
	dcopStub = new PwManagerDCOPif_stub("pwmanager", "pwmanager");
}

PwManagerDCOP::~PwManagerDCOP()
{
	delete dcopStub;
}

QPair<unsigned int, unsigned int> PwManagerDCOP::version()
{
	return dcopStub->version();
}

bool PwManagerDCOP::isRunning()
{
	return dcopStub->isRunning();
}

QStringList PwManagerDCOP::getDocumentList()
{
	return dcopStub->getDocumentList();
}

int PwManagerDCOP::connectToDoc()
{
	return dcopStub->connectToDoc();
}

void PwManagerDCOP::disconnectDoc()
{
	dcopStub->disconnectDoc();
}

void PwManagerDCOP::closeConnectedDoc()
{
	dcopStub->closeConnectedDoc();
}

int PwManagerDCOP::openDocument(const QString &filename)
{
	return dcopStub->openDocument(filename);
}

void PwManagerDCOP::changeMpw()
{
	dcopStub->changeMpw();
}

QStringList PwManagerDCOP::getCategoryList()
{
	return dcopStub->getCategoryList();
}

int PwManagerDCOP::renameCategory(const QString &category,
				  const QString &newName)
{
	return dcopStub->renameCategory(category, newName);
}

int PwManagerDCOP::deleteCategory(const QString &category)
{
	return dcopStub->deleteCategory(category);
}

QStringList PwManagerDCOP::getEntryList(const QString &category)
{
	return dcopStub->getEntryList(category);
}

int PwManagerDCOP::addEntry(const QString &category,
			    const entry_struct &entry)
{
	return dcopStub->addEntry(category,
				  entry.desc,
				  entry.name,
				  entry.pw,
				  entry.comment,
				  entry.url,
				  entry.launcher);
}

void PwManagerDCOP::addEntry_wnd(const QString &pw)
{
	dcopStub->addEntry_wnd(pw);
}

PwManagerDCOP::entry_struct PwManagerDCOP::getEntry(const QString &category, int index)
{
	entry_struct ret;
	ret.clear();
	QStringList tmpRet(dcopStub->getEntry(category, index));
	if (tmpRet.empty() || (tmpRet.size() != 6))
		return ret;
	ret.desc = tmpRet[0];
	ret.name = tmpRet[1];
	ret.pw = tmpRet[2];
	ret.comment = tmpRet[3];
	ret.url = tmpRet[4];
	ret.launcher = tmpRet[5];
	return ret;
}

int PwManagerDCOP::editEntry(const QString &category, int index,
			     const QString &newCategory,
			     const entry_struct &newEntryData)
{
	return dcopStub->editEntry(category, index,
				   newCategory,
				   newEntryData.desc,
				   newEntryData.name,
				   newEntryData.pw,
				   newEntryData.comment,
				   newEntryData.url,
				   newEntryData.launcher);
}

void PwManagerDCOP::editEntry_wnd(const QString &category, int index)
{
	dcopStub->editEntry_wnd(category, index);
}

int PwManagerDCOP::deleteEntry(const QString &category, int index)
{
	return dcopStub->deleteEntry(category, index);
}

#include "../pwmanagerdcopif_stub.cpp"
#include "pwmanagerdcop.moc"
