/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004 by Michael Buesch                            *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "pwmanagerdcopif.h"
#include "pwmexception.h"
#include "listobjselectwnd.h"
#include "pwm.h"
#include "pwmdoc.h"

#define PWMANAGER_DCOP_REV_MAJOR	(static_cast<unsigned int>(0x00))
#define PWMANAGER_DCOP_REV_MINOR	(static_cast<unsigned int>(0x00))


void PwManagerDCOPif::ConnectionList::add(DCOPClient *client, PwMDoc *doc)
{
#ifdef PWM_DEBUG
	// check for existance of object in debug mode only.
	vector< pair<DCOPClient *, PwMDoc *> >::iterator
			begin = connList.begin(),
			end = connList.end(),
			i = begin;
	while (i != end) {
		if (i->first == client) {
			BUG();
			return;
		}
		++i;
	}
#endif
	pair<DCOPClient *, PwMDoc *> newItem(client, doc);
	connList.push_back(newItem);
}

void PwManagerDCOPif::ConnectionList::del(DCOPClient *client)
{
	vector< pair<DCOPClient *, PwMDoc *> >::iterator
			begin = connList.begin(),
			end = connList.end(),
			i = begin;
	while (i != end) {
		if (i->first == client) {
			connList.erase(i);
			return;
		}
		++i;
	}
}

void PwManagerDCOPif::ConnectionList::del(PwMDoc *doc)
{
	vector< pair<DCOPClient *, PwMDoc *> >::iterator
			begin = connList.begin(),
			end = connList.end(),
			i = begin;
	while (i != end) {
		if (i->second == doc) {
			connList.erase(i);
			return;
		}
		++i;
	}
}

PwMDoc * PwManagerDCOPif::ConnectionList::get(DCOPClient *client)
{
	vector< pair<DCOPClient *, PwMDoc *> >::iterator
			begin = connList.begin(),
			end = connList.end(),
			i = begin;
	while (i != end) {
		if (i->first == client) {
			return i->second;
		}
		++i;
	}
	return 0;
}


PwManagerDCOPif::PwManagerDCOPif(const QCString &objId, QWidget *parent,
				 PwM *_mainClass, const char *name)
 : QObject(parent, name)
 , DCOPObject(objId)
{
	parentWidget = parent;
	mainClass = _mainClass;
}

bool PwManagerDCOPif::checkConnection(DCOPClient *client)
{
	if (connectList.get(client))
		return true;
	return false;
}

QPair<unsigned int, unsigned int> PwManagerDCOPif::version()
{
	return qMakePair(PWMANAGER_DCOP_REV_MAJOR,
			 PWMANAGER_DCOP_REV_MINOR);
}

bool PwManagerDCOPif::isRunning()
{
	return true;
}

QStringList PwManagerDCOPif::getDocumentList()
{
	QStringList ret;
	const vector<PwMDocList::listItem> *curDocList = PwMDoc::getOpenDocList()->getList();
	vector<PwMDocList::listItem>::const_iterator begin = curDocList->begin(),
						     end = curDocList->end(),
						     i = begin;
	while (i != end) {
		ret.push_back(i->docId.c_str());
		++i;
	}
	return ret;
}

int PwManagerDCOPif::openDocument(const QString &filename)
{
	PwMDoc *doc = mainClass->openDoc(filename);
	if (!doc)
		return 1;
	doc->setDcopConnFlag(true);
	connectList.add(callingDcopClient(), doc);
	return 0;
}

int PwManagerDCOPif::connectToDoc()
{
	ListObjSelectWnd selWnd(i18n("PwManager: Select document"),
				i18n("Another application has requested "
				     "access to PwManager. Please select "
				     "the document, that you want the application "
				     "would have access to."),
				parentWidget);
	const vector<PwMDocList::listItem> curDocList = *(PwMDoc::getOpenDocList()->getList());
	vector<PwMDocList::listItem>::const_iterator begin = curDocList.begin(),
						     end = curDocList.end(),
						     i = begin;
	while (i != end) {
		selWnd.addObj(i->docId.c_str());
		++i;
	}
	if (selWnd.exec() != 1)
		return -1;
	int selected = selWnd.getSelectedIndex();
	PwMDoc *selDoc = curDocList[selected].doc;
	selDoc->setDcopConnFlag(true);
	connectList.add(callingDcopClient(), selDoc);
	return 0;
}

ASYNC PwManagerDCOPif::disconnectDoc()
{
	PwMDoc *doc = connectList.get(callingDcopClient());
	if (!doc)
		return;
	closeConn(doc);
}

ASYNC PwManagerDCOPif::closeConnectedDoc()
{
	if (!checkConnection(callingDcopClient()))
		return;
	PwMDoc *doc = connectList.get(callingDcopClient());
	/* PwMDoc::closeDoc() triggers PwManagerDCOPif::closeConn() which
	 * closes the DCOP connection, so we don't need to close it here.
	 * (we don't need to remove the connection from the connectList)
	 */
	doc->closeDoc();
}

void PwManagerDCOPif::closeConn(PwMDoc *doc)
{
	connectList.del(doc);
	doc->setDcopConnFlag(false);
}

ASYNC PwManagerDCOPif::changeMpw()
{
	if (!checkConnection(callingDcopClient()))
		return;
	PwMDoc *doc = connectList.get(callingDcopClient());
	doc->changeCurrentPw();
}

QStringList PwManagerDCOPif::getCategoryList()
{
	QStringList ret;
	if (!checkConnection(callingDcopClient()))
		return ret;
	PwMDoc *doc = connectList.get(callingDcopClient());
	vector<string> catVec;
	doc->getCategoryList(&catVec);
	vector<string>::iterator begin = catVec.begin(),
				 end = catVec.end(),
				 i = begin;
	while (i != end) {
		ret.push_back(i->c_str());
		++i;
	}
	return ret;
}

int PwManagerDCOPif::renameCategory(const QString &category, const QString &newName)
{
	if (!checkConnection(callingDcopClient()))
		return -1;
	PwMDoc *doc = connectList.get(callingDcopClient());
	if (!doc->renameCategory(category, newName))
		return 1;
	return 0;
}

int PwManagerDCOPif::deleteCategory(const QString &category)
{
	if (!checkConnection(callingDcopClient()))
		return -1;
	PwMDoc *doc = connectList.get(callingDcopClient());
	if (!doc->delCategory(category))
		return 1;
	return 0;
}

QStringList PwManagerDCOPif::getEntryList(const QString &category)
{
	QStringList ret;
	if (!checkConnection(callingDcopClient()))
		return ret;
	PwMDoc *doc = connectList.get(callingDcopClient());
	vector<string> entrVec;
	doc->getEntryList(category, &entrVec);
	vector<string>::iterator begin = entrVec.begin(),
				 end = entrVec.end(),
				 i = begin;
	while (i != end) {
		ret.push_back(i->c_str());
		++i;
	}
	return ret;
}

int PwManagerDCOPif::addEntry(const QString &category,
			      QString desc, QString name,
			      QString pw, QString comment,
			      QString url, QString launcher)
{
	if (!checkConnection(callingDcopClient()))
		return -1;

	/* XXX: Workaround for some _very_ strange bug, that crashes
	 *      PwManager with the following backtrace. The crash
	 *      occurs, if the DCOP-client doesn't assign a value to
	 *      either struct entry_struct element. Normally, the standard-ctor
	 *      of QString should handle that correct, but here it doesn't
	 *      for some strange reason.

Backtrace:
[New Thread 16384 (LWP 16337)]
[New Thread 32769 (LWP 16338)]
0x4188411a in __libc_waitpid (pid=16370, stat_loc=0x0, options=0)
    at ../sysdeps/unix/sysv/linux/waitpid.c:32
	in ../sysdeps/unix/sysv/linux/waitpid.c
#0  0x4188411a in __libc_waitpid (pid=16370, stat_loc=0x0, options=0)
    at ../sysdeps/unix/sysv/linux/waitpid.c:32
#1  0x40c80ca9 in KCrash::defaultCrashHandler(int) ()
   from /usr/local/kde.cvs/lib/libkdecore.so.4
#2  <signal handler called>
#3  0x41988c03 in strlen () at strlen:-1
#4  0x40e5cbc2 in std::string::operator=(char const*) (this=0xfffffe00, 
    __s=0x0) at char_traits.h:143
#5  0x080b2d9c in PwManagerDCOPif::addEntry(QString const&, QString, QString, QString, QString, QString, QString) (this=0x8203a30, category=@0xbfffecc0, desc=
      {static null = {static null = <same as static member of an already seen type>, d = 0x80e1370, static shared_null = 0x80e1370}, d = 0x8262808, static shared_null = 0x80e1370}, name=
      {static null = {static null = <same as static member of an already seen type>, d = 0x80e1370, static shared_null = 0x80e1370}, d = 0x80e1370, static shared_null = 0x80e1370}, pw=
      {static null = {static null = <same as static member of an already seen type>, d = 0x80e1370, static shared_null = 0x80e1370}, d = 0x80e1370, static shared_null = 0x80e1370}, comment=
      {static null = {static null = <same as static member of an already seen type>, d = 0x80e1370, static shared_null = 0x80e1370}, d = 0x80e1370, static shared_null = 0x80e1370}, url=
      {static null = {static null = <same as static member of an already seen type>, d = 0x80e1370, static shared_null = 0x80e1370}, d = 0x80e1370, static shared_null = 0x80e1370}, launcher=
      {static null = {static null = <same as static member of an already seen type>, d = 0x80e1370, static shared_null = 0x80e1370}, d = 0x80e1370, static shared_null = 0x80e1370}) at pwmanagerdcopif.cpp:268
#6  0x080b5c7f in PwManagerDCOPif::process(QCString const&, QMemArray<char> const&, QCString&, QMemArray<char>&) (this=0x8203a30, fun=@0xbfffef10, 
    data=@0xbfffef40, replyType=@0xbfffef00, replyData=@0xbfffef80)
    at pwmanagerdcopif_skel.cpp:157
#7  0x40d69abd in DCOPClient::receive(QCString const&, QCString const&, QCString const&, QMemArray<char> const&, QCString&, QMemArray<char>&) ()
   from /usr/local/kde.cvs/lib/libDCOP.so.4
#8  0x080de8c0 in vtable for QCString ()
#9  0x00000020 in ?? ()
#10 0x40d6c5cc in DCOPClient::className() const ()
   from /usr/local/kde.cvs/lib/libDCOP.so.4
#11 0xe808245c in ?? ()

	 */
	if (desc.isEmpty())
		desc = "";
	if (name.isEmpty())
		name = "";
	if (pw.isEmpty())
		pw = "";
	if (comment.isEmpty())
		comment = "";
	if (url.isEmpty())
		url = "";
	if (launcher.isEmpty())
		launcher = "";
	/* XXX: END workaround. */


	PwMDoc *doc = connectList.get(callingDcopClient());
	PwMDataItem newItem;
	newItem.clear();
	newItem.desc = desc.latin1();
	newItem.name = name.latin1();
	newItem.pw = pw.latin1();
	newItem.comment = comment.latin1();
	newItem.url = url.latin1();
	newItem.launcher = launcher.latin1();
	PwMerror err;
	err = doc->addEntry(category, &newItem);
	if (err != e_success)
		return 1;
	return 0;
}

QStringList PwManagerDCOPif::getEntry(const QString &category, int index)
{
	QStringList ret;
	if (!checkConnection(callingDcopClient()))
		return ret;
	PwMDoc *doc = connectList.get(callingDcopClient());
	PwMDataItem tmpItem;
	if (!doc->getEntry(category, index, &tmpItem, true))
		return ret;
	ret.push_back(tmpItem.desc.c_str());
	ret.push_back(tmpItem.name.c_str());
	ret.push_back(tmpItem.pw.c_str());
	ret.push_back(tmpItem.comment.c_str());
	ret.push_back(tmpItem.url.c_str());
	ret.push_back(tmpItem.launcher.c_str());
	return ret;
}

int PwManagerDCOPif::editEntry(const QString &category, int index,
			       const QString &newCategory,
			       QString newDesc, QString newName,
			       QString newPw, QString newComment,
			       QString newUrl, QString newLauncher)
{
	if (!checkConnection(callingDcopClient()))
		return -1;

	if (newDesc.isEmpty())
		newDesc = "";
	if (newName.isEmpty())
		newName = "";
	if (newPw.isEmpty())
		newPw = "";
	if (newComment.isEmpty())
		newComment = "";
	if (newUrl.isEmpty())
		newUrl = "";
	if (newLauncher.isEmpty())
		newLauncher = "";

	PwMDoc *doc = connectList.get(callingDcopClient());
	PwMDataItem newItem;
	newItem.clear();
	newItem.desc = newDesc.latin1();
	newItem.name = newName.latin1();
	newItem.pw = newPw.latin1();
	newItem.comment = newComment.latin1();
	newItem.url = newUrl.latin1();
	newItem.launcher = newLauncher.latin1();
	if (!doc->editEntry(category, newCategory, index, &newItem))
		return 1;
	return 0;
}

int PwManagerDCOPif::deleteEntry(const QString &category, int index)
{
	if (!checkConnection(callingDcopClient()))
		return -1;
	PwMDoc *doc = connectList.get(callingDcopClient());
	if (!doc->delEntry(category, index))
		return 1;
	return 0;
}

ASYNC PwManagerDCOPif::addEntry_wnd(QString pw)
{
	if (!checkConnection(callingDcopClient()))
		return;
	PwMDoc *doc = connectList.get(callingDcopClient());
	if (pw == "")
		mainClass->addPwd_slot(0, doc);
	else
		mainClass->addPwd_slot(&pw, doc);
}

ASYNC PwManagerDCOPif::editEntry_wnd(QString category, int index)
{
	if (!checkConnection(callingDcopClient()))
		return;
	PwMDoc *doc = connectList.get(callingDcopClient());
	mainClass->editPwd_slot(&category, &index, doc);
}

#include "pwmanagerdcopif.moc"
