/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004 by Michael Buesch                            *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef PWMANAGERDCOPIF_H
#define PWMANAGERDCOPIF_H

#include <qstring.h>
#include <qstringlist.h>
#include <qpair.h>
#include <qwidget.h>
#include <dcopobject.h>

#include <vector>
#include <utility>

using std::vector;
using std::pair;

class DCOPClient;
class PwM;
class PwMDoc;

/** pwmanager internal DCOP interface class */
class PwManagerDCOPif
	: public QObject
	, public DCOPObject
{
	Q_OBJECT
	K_DCOP

	class ConnectionList
	{
	public:
		ConnectionList() {}
		~ConnectionList() {}

		/** add a new item to the list */
		void add(DCOPClient *client, PwMDoc *doc);
		/** remove the given item */
		void del(DCOPClient *client);
		/** remove the given item */
		void del(PwMDoc *doc);
		/** get item */
		PwMDoc * get(DCOPClient *client);

	protected:
		vector< pair<DCOPClient *, PwMDoc *> > connList;
	};

public:
	PwManagerDCOPif(const QCString &objId, QWidget *parent,
			PwM *_mainClass, const char *name = 0);
	virtual ~PwManagerDCOPif() {}

	/** closes the connection (if there is one) for the given doc */
	void closeConn(PwMDoc *doc);

protected:
	/** checks if the given DCOP client is connected */
	bool checkConnection(DCOPClient *client);

protected:
	/** pointer to the main widget */
	QWidget *parentWidget;
	/** pointer to the main application class */
	PwM *mainClass;
	/** list of all connected clients. */
	ConnectionList connectList;

k_dcop:
	QPair<unsigned int, unsigned int> version();
	bool isRunning();
	int connectToDoc();
	int openDocument(const QString &filename);
	QStringList getDocumentList();
	ASYNC disconnectDoc();
	ASYNC closeConnectedDoc();
	ASYNC changeMpw();
	QStringList getCategoryList();
	int renameCategory(const QString &category, const QString &newName);
	int deleteCategory(const QString &category);
	QStringList getEntryList(const QString &category);
	int addEntry(const QString &category,
		     QString desc, QString name,
		     QString pw, QString comment,
		     QString url, QString launcher);
	/* @return Returns a QStringList with the contents of the
	 *         given entry. The items of the entry are sorted
	 *         as in PwManagerDCOP::entry_struct
	 */
	QStringList getEntry(const QString &category, int index);
	int editEntry(const QString &category, int index,
		      const QString &newCategory,
		      QString newDesc, QString newName,
		      QString newPw, QString newComment,
		      QString newUrl, QString newLauncher);
	int deleteEntry(const QString &category, int index);
	ASYNC addEntry_wnd(QString pw);
	ASYNC editEntry_wnd(QString category, int index);
};

#endif
