/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004 by Michael Buesch                                  *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "genpasswd.h"
#include "pwmexception.h"
#include "configuration.h"


GenPasswd::GenPasswd()
{
	curGen = -1;
	connect(&genProc, SIGNAL(receivedStdout(KProcess *, char *, int)),
		this, SLOT(receivedStdout(KProcess *, char *, int)));
}

GenPasswd::~GenPasswd()
{
}

void GenPasswd::gen()
{
	int generator = conf()->confGlobPwGen();
	switch (generator) {
	case 0: // makepasswd
		curGen = 0;
		gen_makepasswd();
		break;
	case 1: // PwG
		curGen = 1;
		gen_pwg();
		break;
	default:
		curGen = -1;
		emit pwErr(GEN_ERR_PROGNOTFOUND);
		break;
	}
}

void GenPasswd::gen_makepasswd()
{
	/* NOTE: There are (at least) two versions of makepasswd
	 * around. (one C and one perl version). We have to be
	 * compatible with _both_.
	 * Perl version:  Available as debian package "makepasswd"
	 * C version:     Available here: http://www.defora.org/index.php?page=projects&project=makepasswd
	 */
	genProc.clearArguments();
	genProc << "makepasswd";
	TODO();
	genProc << "-l" << "10"; //TODO: make length user-configurable
	genProc.start(KProcess::NotifyOnExit, KProcess::Stdout);
}

void GenPasswd::gen_pwg()
{//TODO
	TODO();
	emit pwErr(GEN_ERR_PROGNOTFOUND);
	curGen = -1;
}

void GenPasswd::receivedStdout(KProcess * /*proc*/,
			       char *buffer,
			       int buflen)
{
	QString buf = QString::fromLatin1(buffer, buflen);
	genProc.kill();
	emit pwAvailable(buf);
}

#include "genpasswd.moc"
