/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004 by Michael Buesch                                  *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef GENPASSWD_H
#define GENPASSWD_H

#include <qobject.h>

#include <kprocess.h>

#define GEN_ERR_PROGNOTFOUND	-1

/** This class generates random passwords using
  * external generator programs.
  */
class GenPasswd : public QObject
{
	Q_OBJECT
public:
	GenPasswd();
	~GenPasswd();

	/** generate a new password. If it's available
	  * the signal pwAvailable(QString) is emitted.
	  * If an error occured signal pwErr(int) is emitted.
	  */
	void gen();

signals:
	/** the generated password is available */
	void pwAvailable(const QString &pw);
	/** an error while generation occured */
	void pwErr(int);

protected:
	/** generate a new password with "makepasswd" */
	void gen_makepasswd();
	/** generatr a new password with "PwG" */
	void gen_pwg();

protected slots:
	void receivedStdout(KProcess *proc, char *buffer,
			    int buflen);

protected:
	/** current generator used.
	  * -1 -> none
	  * 0  -> makepasswd
	  * 1  -> PwG
	  */
	int curGen;
	/** process object */
	KProcess genProc;
};

#endif
