/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004 by Michael Buesch                                  *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   KWallet DCOP Interface is                                             *
 *    Copyright (c) 2002-2003 George Staikos <staikos@kde.org>             *
 *   KWallet is part of the KDE libraries                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "kded_kwalletemu.h"
#include "kwalletemu_stub.h"
#include "kwalletemu_stub.cpp"
#include "globalstuff.h"

#include <kdatastream.h>
#include <kapplication.h>

#include <dcopclient.h>
#include <dcopref.h>


#define EMULATE_KWALLET_NAME	"kwalletd"

#if defined(PWM_DEBUG) && KDED_KWALLETEMU_DEBUG != 0
# include <string>
  using std::string;
# define prDbg(msg)	system((string("logger PwManager-KWalletEmu: ") \
				+ string(msg)).c_str())
#else
# define prDbg(msg)	do {} while (0)
#endif


extern "C"
{
	KDEDModule * create_pwmanager_kwalletemu(const QCString &)
	{
		return (new Kded_KWalletEmu(EMULATE_KWALLET_NAME));
	}
}


Kded_KWalletEmu::Kded_KWalletEmu(const QCString &name)
 : KDEDModule(name)
{
	const char *pwmanager_app = "pwmanager";
	const char *pwmanager_obj = "kwalletemu";
	dcopClient = kapp->dcopClient();
	stub = new KWalletEmu_stub(pwmanager_app, pwmanager_obj);

	dcopClient->connectDCOPSignal(pwmanager_app, pwmanager_obj,
				      "walletClosed(QString)",
				      EMULATE_KWALLET_NAME,
				      "slotWalletClosed(QString)",
				      false);
	dcopClient->connectDCOPSignal(pwmanager_app, pwmanager_obj,
				      "walletClosed(int)",
				      EMULATE_KWALLET_NAME,
				      "slotWalletClosed(int)",
				      false);
	dcopClient->connectDCOPSignal(pwmanager_app, pwmanager_obj,
				      "allWalletsClosed()",
				      EMULATE_KWALLET_NAME,
				      "slotAllWalletsClosed()",
				      false);
	dcopClient->connectDCOPSignal(pwmanager_app, pwmanager_obj,
				      "applicationDisconnected(QString, QCString)",
				      EMULATE_KWALLET_NAME,
				      "slotApplicationDisconnected(QString, QCString)",
				      false);
	dcopClient->connectDCOPSignal(pwmanager_app, pwmanager_obj,
				      "folderUpdated(QString, QString)",
				      EMULATE_KWALLET_NAME,
				      "slotFolderUpdated(QString, QString)",
				      false);
	dcopClient->connectDCOPSignal(pwmanager_app, pwmanager_obj,
				      "folderListUpdated(QString)",
				      EMULATE_KWALLET_NAME,
				      "slotFolderListUpdated(QString)",
				      false);
	dcopClient->connectDCOPSignal(pwmanager_app, pwmanager_obj,
				      "walletListDirty()",
				      EMULATE_KWALLET_NAME,
				      "slotWalletListDirty()",
				      false);
	dcopClient->connectDCOPSignal(pwmanager_app, pwmanager_obj,
				      "walletCreated(QString)",
				      EMULATE_KWALLET_NAME,
				      "slotWalletCreated(QString)",
				      false);
	dcopClient->connectDCOPSignal(pwmanager_app, pwmanager_obj,
				      "walletOpened(QString)",
				      EMULATE_KWALLET_NAME,
				      "slotWalletOpened(QString)",
				      false);
	dcopClient->connectDCOPSignal(pwmanager_app, pwmanager_obj,
				      "walletDeleted(QString)",
				      EMULATE_KWALLET_NAME,
				      "slotWalletDeleted(QString)",
				      false);
	prDbg("initialized");
}

Kded_KWalletEmu::~Kded_KWalletEmu()
{
	delete stub;
}

QCString Kded_KWalletEmu::DCOPPeerName()
{
	DCOPClient *dc = callingDcopClient();
	if (!dc)
		return "";
	return dc->senderId();
}

bool Kded_KWalletEmu::isEnabled() const
{
	return stub->isEnabled();
}

int Kded_KWalletEmu::open(const QString& wallet,
			  uint wId)
{
	return stub->open(wallet, wId);
}

int Kded_KWalletEmu::openPath(const QString& path,
			      uint wId)
{
	return stub->openPath(path, wId);
}

void Kded_KWalletEmu::openAsynchronous(const QString& wallet,
				       const QCString& returnObject,
				       uint wId)
{
	stub->openAsynchronous(wallet, returnObject, DCOPPeerName(), wId);
}

int Kded_KWalletEmu::close(const QString& wallet,
			   bool force)
{
	return stub->close(wallet, force);
}

int Kded_KWalletEmu::close(int handle,
			   bool force)
{
	return stub->close(handle, force);
}

ASYNC Kded_KWalletEmu::sync(int handle)
{
	stub->sync(handle);
}

int Kded_KWalletEmu::deleteWallet(const QString& wallet)
{
	return stub->deleteWallet(wallet);
}

bool Kded_KWalletEmu::isOpen(const QString& wallet) const
{
	return stub->isOpen(wallet);
}

bool Kded_KWalletEmu::isOpen(int handle)
{
	return stub->isOpen(handle);
}

QStringList Kded_KWalletEmu::users(const QString& wallet) const
{
	return stub->users(wallet);
}

void Kded_KWalletEmu::changePassword(const QString& wallet,
				     uint wId)
{
	stub->changePassword(wallet, wId);
}

QStringList Kded_KWalletEmu::wallets() const
{
	return stub->wallets();
}

QStringList Kded_KWalletEmu::folderList(int handle)
{
	return stub->folderList(handle);
}

bool Kded_KWalletEmu::hasFolder(int handle,
				const QString& folder)
{
	return stub->hasFolder(handle, folder);
}

bool Kded_KWalletEmu::createFolder(int handle,
				   const QString& folder)
{
	return stub->createFolder(handle, folder);
}

bool Kded_KWalletEmu::removeFolder(int handle,
				   const QString& folder)
{
	return stub->removeFolder(handle, folder);
}

QStringList Kded_KWalletEmu::entryList(int handle,
				       const QString& folder)
{
	return stub->entryList(handle, folder);
}

QByteArray Kded_KWalletEmu::readEntry(int handle,
				      const QString& folder,
				      const QString& key)
{
	return stub->readEntry(handle, folder, key);
}

QByteArray Kded_KWalletEmu::readMap(int handle,
				    const QString& folder,
				    const QString& key)
{
	return stub->readMap(handle, folder, key);
}

QString Kded_KWalletEmu::readPassword(int handle,
				      const QString& folder,
				      const QString& key)
{
	return stub->readPassword(handle, folder, key);
}

int Kded_KWalletEmu::renameEntry(int handle,
				 const QString& folder,
				 const QString& oldName,
				 const QString& newName)
{
	return stub->renameEntry(handle, folder, oldName, newName);
}

int Kded_KWalletEmu::writeEntry(int handle,
				const QString& folder,
				const QString& key,
				const QByteArray& value,
				int entryType)
{
	return stub->writeEntry(handle, folder, key, value, entryType);
}

int Kded_KWalletEmu::writeEntry(int handle,
				const QString& folder,
				const QString& key,
				const QByteArray& value)
{
	return stub->writeEntry(handle, folder, key, value);
}

int Kded_KWalletEmu::writeMap(int handle,
			      const QString& folder,
			      const QString& key,
			      const QByteArray& value)
{
	return stub->writeMap(handle, folder, key, value);
}

int Kded_KWalletEmu::writePassword(int handle,
				   const QString& folder,
				   const QString& key,
				   const QString& value)
{
	return stub->writePassword(handle, folder, key, value);
}

bool Kded_KWalletEmu::hasEntry(int handle,
			       const QString& folder,
			       const QString& key)
{
	return stub->hasEntry(handle, folder, key);
}

int Kded_KWalletEmu::entryType(int handle,
			       const QString& folder,
			       const QString& key)
{
	return stub->entryType(handle, folder, key);
}

int Kded_KWalletEmu::removeEntry(int handle,
				 const QString& folder,
				 const QString& key)
{
	return stub->removeEntry(handle, folder, key);
}

bool Kded_KWalletEmu::disconnectApplication(const QString& wallet,
					    const QCString& application)
{
	return stub->disconnectApplication(wallet, application);
}

void Kded_KWalletEmu::reconfigure()
{
	stub->reconfigure();
}

bool Kded_KWalletEmu::folderDoesNotExist(const QString& wallet,
					 const QString& folder)
{
	return stub->folderDoesNotExist(wallet, folder);
}

bool Kded_KWalletEmu::keyDoesNotExist(const QString& wallet,
				      const QString& folder,
				      const QString& key)
{
	return stub->keyDoesNotExist(wallet, folder, key);
}

void Kded_KWalletEmu::closeAllWallets()
{
	stub->closeAllWallets();
}

bool Kded_KWalletEmu::isEmulated() const
{
	return true;
}

QString Kded_KWalletEmu::emulationApp() const
{
	return "pwmanager";
}

ASYNC Kded_KWalletEmu::slotWalletClosed(const QString &wallet)
{
	prDbg("DCOP-signal: walletClosed QString");
	QByteArray b;
	QDataStream ds(b, IO_WriteOnly);
	ds << wallet;
	dcopClient->emitDCOPSignal(EMULATE_KWALLET_NAME,
				   "walletClosed(QString)", b);
}

ASYNC Kded_KWalletEmu::slotWalletClosed(int handle)
{
	prDbg("DCOP-signal: walletClosed int");
	QByteArray b;
	QDataStream ds(b, IO_WriteOnly);
	ds << handle;
	dcopClient->emitDCOPSignal(EMULATE_KWALLET_NAME,
				   "walletClosed(int)", b);
}

ASYNC Kded_KWalletEmu::slotAllWalletsClosed()
{
	prDbg("DCOP-signal: allWalletsClosed");
	dcopClient->emitDCOPSignal(EMULATE_KWALLET_NAME,
				   "allWalletClosed()", QByteArray());
}

ASYNC Kded_KWalletEmu::slotApplicationDisconnected(const QString &wallet,
						   const QCString &application)
{
	prDbg("DCOP-signal: applicationDisconnected");
	QByteArray b;
	QDataStream ds(b, IO_WriteOnly);
	ds << wallet;
	ds << application;
	dcopClient->emitDCOPSignal(EMULATE_KWALLET_NAME,
				   "applicationDisconnected(QString,QCString)", b);
}

ASYNC Kded_KWalletEmu::slotFolderUpdated(const QString &wallet,
					 const QString &folder)
{
	prDbg("DCOP-signal: folderUpdated");
	QByteArray b;
	QDataStream ds(b, IO_WriteOnly);
	ds << wallet;
	ds << folder;
	dcopClient->emitDCOPSignal(EMULATE_KWALLET_NAME,
				   "folderUpdated(QString,QString)", b);
}

ASYNC Kded_KWalletEmu::slotFolderListUpdated(const QString &wallet)
{
	prDbg("DCOP-signal: folderListUpdated");
	QByteArray b;
	QDataStream ds(b, IO_WriteOnly);
	ds << wallet;
	dcopClient->emitDCOPSignal(EMULATE_KWALLET_NAME,
				   "folderListUpdated(QString)", b);
}

ASYNC Kded_KWalletEmu::slotWalletListDirty()
{
	prDbg("DCOP-signal: walletListDirty");
	dcopClient->emitDCOPSignal(EMULATE_KWALLET_NAME,
				   "walletListDirty()", QByteArray());
}

ASYNC Kded_KWalletEmu::slotWalletCreated(const QString &wallet)
{
	prDbg("DCOP-signal: walletCreated");
	QByteArray b;
	QDataStream ds(b, IO_WriteOnly);
	ds << wallet;
	dcopClient->emitDCOPSignal(EMULATE_KWALLET_NAME,
				   "walletCreated(QString)", b);
}

ASYNC Kded_KWalletEmu::slotWalletOpened(const QString &wallet)
{
	prDbg("DCOP-signal: walletOpened");
	QByteArray b;
	QDataStream ds(b, IO_WriteOnly);
	ds << wallet;
	dcopClient->emitDCOPSignal(EMULATE_KWALLET_NAME,
				   "walletOpened(QString)", b);
}

ASYNC Kded_KWalletEmu::slotWalletDeleted(const QString &wallet)
{
	prDbg("DCOP-signal: walletDeleted");
	QByteArray b;
	QDataStream ds(b, IO_WriteOnly);
	ds << wallet;
	dcopClient->emitDCOPSignal(EMULATE_KWALLET_NAME,
				   "walletDeleted(QString)", b);
}

ASYNC Kded_KWalletEmu::walletOpenResult(int rc,
					const QCString &returnObject,
					const QCString &returnApp)
{
	prDbg("walletOpenResult callback");
	DCOPRef(returnApp, returnObject).send("walletOpenResult", rc);
}

#include "kded_kwalletemu_skel.cpp"
#include "kded_kwalletemu.moc"
