/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004 by Michael Buesch                                  *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   KWallet DCOP Interface is                                             *
 *    Copyright (c) 2002-2003 George Staikos <staikos@kde.org>             *
 *   KWallet is part of the KDE libraries                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef KDED_KWALLETEMU_H
#define KDED_KWALLETEMU_H

#include <kdedmodule.h>
#include <dcopclient.h>
#include <qcstring.h>

/* enable/disable Kded_KwalletEmu debugging */
#define KDED_KWALLETEMU_DEBUG	0

class KWalletEmu_stub;

/** This is the KDED kwalletd emulation layer module
  * for PwManager.
  * This module maps all KWallet DCOP calls on
  * the PwManager KWallet emulation interface.
  */
class Kded_KWalletEmu : public KDEDModule
{
	Q_OBJECT
	K_DCOP
public:
	Kded_KWalletEmu(const QCString &name);
	virtual ~Kded_KWalletEmu();

protected:
	/** stub object of _real_ emulation interface */
	KWalletEmu_stub *stub;
	/** DCOP client object */
	DCOPClient *dcopClient;

protected:
	QCString DCOPPeerName();

k_dcop:
/* KWallet interface emulation */
	virtual bool isEnabled() const;
	virtual int open(const QString& wallet, uint wId);
	virtual int openPath(const QString& path, uint wId);
	virtual void openAsynchronous(const QString& wallet,
				      const QCString& returnObject,
				      uint wId);
	virtual int close(const QString& wallet, bool force);
	virtual int close(int handle, bool force);
	virtual ASYNC sync(int handle);
	virtual int deleteWallet(const QString& wallet);
	virtual bool isOpen(const QString& wallet) const;
	virtual bool isOpen(int handle);
	virtual QStringList users(const QString& wallet) const;
	virtual void changePassword(const QString& wallet, uint wId);
	virtual QStringList wallets() const;
	virtual QStringList folderList(int handle);
	virtual bool hasFolder(int handle, const QString& folder);
	virtual bool createFolder(int handle, const QString& folder);
	virtual bool removeFolder(int handle, const QString& folder);
	virtual QStringList entryList(int handle,
				      const QString& folder);
	virtual QByteArray readEntry(int handle,
				     const QString& folder,
				     const QString& key);
	virtual QByteArray readMap(int handle,
				   const QString& folder,
				   const QString& key);
	virtual QString readPassword(int handle,
				     const QString& folder,
				     const QString& key);
	virtual int renameEntry(int handle,
				const QString& folder,
				const QString& oldName,
				const QString& newName);
	virtual int writeEntry(int handle,
			       const QString& folder,
			       const QString& key,
			       const QByteArray& value,
			       int entryType);
	virtual int writeEntry(int handle,
			       const QString& folder,
			       const QString& key,
			       const QByteArray& value);
	virtual int writeMap(int handle,
			     const QString& folder,
			     const QString& key,
			     const QByteArray& value);
	virtual int writePassword(int handle,
				  const QString& folder,
				  const QString& key,
				  const QString& value);
	virtual bool hasEntry(int handle,
			      const QString& folder,
			      const QString& key);
	virtual int entryType(int handle,
			      const QString& folder,
			      const QString& key);
	virtual int removeEntry(int handle,
				const QString& folder,
				const QString& key);
	virtual bool disconnectApplication(const QString& wallet,
					   const QCString& application);
	virtual void reconfigure();
	virtual bool folderDoesNotExist(const QString& wallet,
					const QString& folder);
	virtual bool keyDoesNotExist(const QString& wallet,
				     const QString& folder,
				     const QString& key);
	virtual void closeAllWallets();

/* slots for signal forwarding */
	virtual ASYNC slotWalletClosed(const QString &wallet);
	virtual ASYNC slotWalletClosed(int handle);
	virtual ASYNC slotAllWalletsClosed();
	virtual ASYNC slotApplicationDisconnected(const QString &wallet,
						  const QCString &application);
	virtual ASYNC slotFolderUpdated(const QString &wallet,
					const QString &folder);
	virtual ASYNC slotFolderListUpdated(const QString &wallet);
	virtual ASYNC slotWalletListDirty();
	virtual ASYNC slotWalletCreated(const QString &wallet);
	virtual ASYNC slotWalletOpened(const QString &wallet);
	virtual ASYNC slotWalletDeleted(const QString &wallet);

/* callbacks */
	virtual ASYNC walletOpenResult(int rc, const QCString &returnObject,
				       const QCString &returnApp);

/* additional DCOP functions */
	virtual bool isEmulated() const;
	virtual QString emulationApp() const;
};

#endif
