/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004 by Michael Buesch                            *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kuniqueapplication.h>

#include "pwm.h"
#include "pwmexception.h"
#include "selftest.h"

#define LICENSE_FILE	(::locate("data", "pwmanager/pwmanager_license_text"))

static const char *description = I18N_NOOP("PwManager\n"
					   "The convenient way of managing passwords");

static KCmdLineOptions options[] = {
	{0, 0, 0}
};

#ifdef PWM_DEBUG
static void printDebugConfigureInfo()
{
	cout << "================================" << endl;
	cout << PROG_NAME " version " VERSION << endl;
#ifdef CONFIG_KEYCARD
	cout << "CONFIG_KEYCARD:    enabled" << endl;
#else
	cout << "CONFIG_KEYCARD:    disabled" << endl;
#endif
#ifdef CONFIG_KWALLETIF
	cout << "CONFIG_KWALLETIF:  enabled" << endl;
#else
	cout << "CONFIG_KWALLETIF:  disabled" << endl;
#endif
#ifdef BIG_ENDIAN_HOST
	cout << "Endianess:         big-endian" << endl;
#else
	cout << "Endianess:         little-endian" << endl;
#endif
	cout << "sizeof(long):      " << sizeof(long) << endl;
	cout << "================================" << endl;
}
#else // PWM_DEBUG
static inline void printDebugConfigureInfo() { /* nothing */ }
#endif // PWM_DEBUG

int main(int argc, char *argv[])
{
	printDebugConfigureInfo();
	KAboutData aboutData(PACKAGE, PROG_NAME,
			     VERSION, description, KAboutData::License_File,
			     "(c) 2003, 2004 Michael Buesch and the PwManager Team", 0,
			     "http://passwordmanager.sourceforge.net/",
			     "mbuesch@freenet.de");
	aboutData.addAuthor("Michael Buesch",
			    I18N_NOOP
			    ("main programming and current maintainer"),
			    "mbuesch@freenet.de");
	aboutData.addAuthor("Matt Scifo",
			    I18N_NOOP("original implementaion of \n"
				      "\"categories\" and the password-tree \n"
				      "in the system-tray. Original implementations of \n"
				      "numerous view-improvements."),
			    "mscifo@o1.com");
	aboutData.addAuthor("Wickey", I18N_NOOP("graphics-design"),
			    "wickey@gmx.at");
	aboutData.addCredit("Elias Probst",
			    "inofficial Gentoo ebuild maintainer.\n"
			    "Elias builds the Gentoo ebuilds.\n"
			    "Please send all e-mail about ebuilds\n"
			    "to Michael Buesch <mbuesch@freenet.de>.");
	aboutData.addCredit("George Staikos",
			    "KWallet", "staikos@kde.org");
	aboutData.addCredit("Matthew Palmer", "rc2 code",
			    "mjp16@uow.edu.au");
        aboutData.addCredit("Olivier Sessink", "gpasman",
			    "gpasman@nl.linux.org");
	aboutData.addCredit("The libgcrypt developers", "Blowfish and SHA1 algorithms",
			    0, "ftp://ftp.gnupg.org/gcrypt/alpha/libgcrypt/");
	aboutData.addCredit("Troy Engel", "kpasman",
			    "tengel@sonic.net");

	KCmdLineArgs::init(argc, argv, &aboutData);
	KCmdLineArgs::addCmdLineOptions(options);

	KUniqueApplication::addCmdLineOptions();
	if (!KUniqueApplication::start()) {
		printInfo("already running.");
		return EXIT_SUCCESS;
	}
	KUniqueApplication a;
	aboutData.setLicenseTextFile(LICENSE_FILE);
	earlyGlobalInit();
	(new SelfTest)->schedule();
	a.setMainWidget(new PwM);
	int ret = a.exec();
	lateGlobalCleanup();
	return ret;
}
