/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004 by Michael Buesch                            *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>
#include <kmenubar.h>
#include <klistview.h>
#include <ktoolbar.h>
#include <kfiledialog.h>
#include <kstatusbar.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kstatusbar.h>

#include <qpixmap.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qfileinfo.h>
#include <qclipboard.h>

#include <dcopclient.h>

#include <stdio.h>

#include "pwm.h"
#include "configwndimpl.h"
#include "pwmprint.h"
#include "addentrywndimpl.h"
#include "globalstuff.h"
#include "pwmanagerdcopif.h"
#include "findwndimpl.h"
#include "configuration.h"

#ifdef CONFIG_KWALLETIF
# include "kwalletif.h"
# include "kwalletemu.h"
#endif
#ifdef CONFIG_KEYCARD
# include "pwmkeycard.h"
#endif

// name of PwG binary executable
#define PWG_BIN_NAME		"pwg"


// Button IDs for "file" popup menu
enum {
	BUTTON_POPUP_FILE_NEW = 0,
	BUTTON_POPUP_FILE_OPEN,
	BUTTON_POPUP_FILE_CLOSE,
	BUTTON_POPUP_FILE_SAVE,
	BUTTON_POPUP_FILE_SAVEAS,
	BUTTON_POPUP_FILE_EXPORT,
	BUTTON_POPUP_FILE_IMPORT,
	BUTTON_POPUP_FILE_PRINT,
	BUTTON_POPUP_FILE_QUIT
};
// Button IDs for "manage" popup menu
enum {
	BUTTON_POPUP_MANAGE_ADD = 0,
	BUTTON_POPUP_MANAGE_EDIT,
	BUTTON_POPUP_MANAGE_DEL,
	BUTTON_POPUP_MANAGE_CHANGEMP,
	BUTTON_POPUP_MANAGE_PWG
};
// Button IDs for chipcard popup menu
enum {
#ifdef CONFIG_KEYCARD
	BUTTON_POPUP_CHIPCARD_GENNEW = 0,
	BUTTON_POPUP_CHIPCARD_DEL,
	BUTTON_POPUP_CHIPCARD_READID,
	BUTTON_POPUP_CHIPCARD_SAVEBACKUP,
	BUTTON_POPUP_CHIPCARD_REPLAYBACKUP
#else // CONFIG_KEYCARD
	BUTTON_POPUP_CHIPCARD_NO = 0
#endif // CONFIG_KEYCARD
};
// Button IDs for "view" popup menu
enum {
	BUTTON_POPUP_VIEW_FIND = 0,
	BUTTON_POPUP_VIEW_LOCK,
	BUTTON_POPUP_VIEW_DEEPLOCK,
	BUTTON_POPUP_VIEW_UNLOCK
};
// Button IDs for "options" popup menu
enum {
	BUTTON_POPUP_OPTIONS_CONFIG = 0
};
// Button IDs for "export" popup menu (in "file" popup menu)
enum {
	BUTTON_POPUP_EXPORT_TEXT = 0,
	BUTTON_POPUP_EXPORT_GPASMAN
#ifdef CONFIG_KWALLETIF
	,BUTTON_POPUP_EXPORT_KWALLET
#endif
};
// Button IDs for "import" popup menu (in "file" popup menu)
enum {
	BUTTON_POPUP_IMPORT_TEXT = 0,
	BUTTON_POPUP_IMPORT_GPASMAN
#ifdef CONFIG_KWALLETIF
	,BUTTON_POPUP_IMPORT_KWALLET
#endif
};
// Button IDs for toolbar
enum {
	BUTTON_TOOL_NEW = 0,
	BUTTON_TOOL_OPEN,
	BUTTON_TOOL_SAVE,
	BUTTON_TOOL_SAVEAS,
	BUTTON_TOOL_PRINT,
	BUTTON_TOOL_ADD,
	BUTTON_TOOL_EDIT,
	BUTTON_TOOL_DEL,
	BUTTON_TOOL_FIND,
	BUTTON_TOOL_LOCK,
	BUTTON_TOOL_DEEPLOCK,
	BUTTON_TOOL_UNLOCK,
	BUTTON_TOOL_PWG
};


PwM::PwM(QWidget *parent, const char *name)
 : KMainWindow(parent, name)
{
	bool showMainWnd = true;

	pwgproc << PWG_BIN_NAME;

#ifdef CONFIG_KEYCARD
	keyCard = new PwMKeyCard(this);
#endif
	workspace = new QWorkspace(this);
	setCentralWidget(workspace);
	connect(workspace, SIGNAL(windowActivated(QWidget *)), this,
		SLOT(wndActivated_slot(QWidget *)));

	initMenubar();
	initToolbar();
	initMetrics();
	currBg = 0;
	initBackgroundPix();

	tray = 0;
	if (conf()->confGlobTray()) {
		showTray();
		if (conf()->confWndAutoMinimizeOnStart())
			showMainWnd = false;
	}

	setCaption(QString("PassWordManager - ") + i18n("version")
		   + " " VERSION);
	currentView = 0;
	enableMdiUi(false);
	statusBar()->show();
	if (showMainWnd)
		show();

	dcopIf = 0;
	initDCOP();
#ifdef CONFIG_KWALLETIF
	kwalletEmu = 0;
	initKWalletEmu();
#endif
	// open autostart-file (if present)
	QString autoStart(conf()->confGlobAutoStart());
	if (autoStart != "")
		openDoc(autoStart, conf()->confGlobAutostartDeepLocked());
	showStatMsg(i18n("Ready."));
}

PwM::~PwM()
{
	hideTray();
	conf()->confWndMainWndSize(size());
	delete_ifnot_null(currBg);
#ifdef CONFIG_KEYCARD
	delete_ifnot_null(keyCard);
#endif
#ifdef CONFIG_KWALLETIF
	delete_ifnot_null(kwalletEmu);
#endif
}

void PwM::initMenubar()
{
	KIconLoader icons;

	filePopup = new KPopupMenu(this);
	importPopup = new KPopupMenu(filePopup);
	exportPopup = new KPopupMenu(filePopup);
	managePopup = new KPopupMenu(this);
	chipcardPopup = new KPopupMenu(this);
	viewPopup = new KPopupMenu(this);
	windowPopup = new KPopupMenu(this);
	optionsPopup = new KPopupMenu(this);

// "file" popup menu
	filePopup->insertItem(QIconSet(icons.loadIcon("filenew", KIcon::Small)),
			      i18n("&New"), this,
			      SLOT(new_slot()), 0, BUTTON_POPUP_FILE_NEW);
	filePopup->insertItem(QIconSet(icons.loadIcon("fileopen", KIcon::Small)),
			      i18n("&Open"), this,
			      SLOT(open_slot()), 0, BUTTON_POPUP_FILE_OPEN);
	filePopup->insertItem(QIconSet(icons.loadIcon("fileclose", KIcon::Small)),
			      i18n("&Close"), this,
			      SLOT(close_slot()), 0, BUTTON_POPUP_FILE_CLOSE);
	filePopup->insertSeparator();
	filePopup->insertItem(QIconSet(icons.loadIcon("filesave", KIcon::Small)),
			      i18n("&Save"), this,
			      SLOT(save_slot()), 0, BUTTON_POPUP_FILE_SAVE);
	filePopup->insertItem(QIconSet(icons.loadIcon("filesaveas", KIcon::Small)),
			      i18n("Save &as..."),
			      this, SLOT(saveAs_slot()), 0,
			      BUTTON_POPUP_FILE_SAVEAS);
	filePopup->insertSeparator();
	// "file/export" popup menu
	exportPopup->insertItem(i18n("&Text-file..."), this,
				SLOT(exportToText()), 0, BUTTON_POPUP_EXPORT_TEXT);
	exportPopup->insertItem(i18n("&Gpasman / Kpasman ..."), this,
				SLOT(exportToGpasman()), 0, BUTTON_POPUP_EXPORT_GPASMAN);
#ifdef CONFIG_KWALLETIF
	exportPopup->insertItem(i18n("&KWallet..."), this,
				SLOT(exportToKWallet()), 0, BUTTON_POPUP_EXPORT_KWALLET);
#endif
	filePopup->insertItem(QIconSet(icons.loadIcon("fileexport", KIcon::Small)),
			      i18n("E&xport"), exportPopup,
			      BUTTON_POPUP_FILE_EXPORT);
	// "file/import" popup menu
	importPopup->insertItem(i18n("&Text-file..."), this,
				SLOT(importFromText()), 0, BUTTON_POPUP_IMPORT_TEXT);
	importPopup->insertItem(i18n("&Gpasman / Kpasman ..."), this,
				SLOT(importFromGpasman()), 0, BUTTON_POPUP_IMPORT_GPASMAN);
#ifdef CONFIG_KWALLETIF
	importPopup->insertItem(i18n("&KWallet..."), this,
				SLOT(importKWallet()), 0, BUTTON_POPUP_IMPORT_KWALLET);
#endif
	filePopup->insertItem(QIconSet(icons.loadIcon("fileimport", KIcon::Small)),
			      i18n("I&mport"), importPopup,
			      BUTTON_POPUP_FILE_IMPORT);
	filePopup->insertSeparator();
	filePopup->insertItem(QIconSet(icons.loadIcon("fileprint", KIcon::Small)),
			      i18n("&Print..."), this,
			      SLOT(print_slot()), 0, BUTTON_POPUP_FILE_PRINT);
	filePopup->insertSeparator();
	filePopup->insertItem(QIconSet(icons.loadIcon("exit", KIcon::Small)),
			      i18n("&Quit"), this,
			      SLOT(close()), 0, BUTTON_POPUP_FILE_QUIT);
	menuBar()->insertItem(i18n("&File"), filePopup);
// "manage" popup menu
	managePopup->insertItem(QIconSet(icons.loadIcon("pencil", KIcon::Small)),
				i18n("&Add password"), this,
				SLOT(addPwd_slot()), 0,
				BUTTON_POPUP_MANAGE_ADD);
	managePopup->insertItem(QIconSet(icons.loadIcon("edit", KIcon::Small)),
				i18n("&Edit"), this, SLOT(editPwd_slot()), 0,
				BUTTON_POPUP_MANAGE_EDIT);
	managePopup->insertItem(QIconSet(icons.loadIcon("editdelete", KIcon::Small)),
				i18n("&Delete"), this, SLOT(deletePwd_slot()),
				0, BUTTON_POPUP_MANAGE_DEL);
	managePopup->insertSeparator();
	managePopup->insertItem(QIconSet(icons.loadIcon("rotate", KIcon::Small)),
				i18n("Change current &master-password"), this,
				SLOT(changeMasterPwd_slot()), 0,
				BUTTON_POPUP_MANAGE_CHANGEMP);
	managePopup->insertSeparator();
	managePopup->insertItem(QIconSet(icons.loadIcon("pwg", KIcon::Small)),
				i18n("Start &PwG..."), this,
				SLOT(startPwG_slot()), 0,
				BUTTON_POPUP_MANAGE_PWG);
	menuBar()->insertItem(i18n("&Manage"), managePopup);
// "chipcard" popup menu
#ifdef CONFIG_KEYCARD
	chipcardPopup->insertItem(QIconSet(icons.loadIcon("filenew", KIcon::Small)),
				  i18n("&Generate new key-card"), this,
				  SLOT(genNewCard_slot()), 0,
				  BUTTON_POPUP_CHIPCARD_GENNEW);
	chipcardPopup->insertItem(QIconSet(icons.loadIcon("editdelete", KIcon::Small)),
				  i18n("&Erase key-card"), this,
				  SLOT(eraseCard_slot()), 0,
				  BUTTON_POPUP_CHIPCARD_DEL);
	chipcardPopup->insertItem(QIconSet(icons.loadIcon("", KIcon::Small)),
				  i18n("Read card-&ID"), this,
				  SLOT(readCardId_slot()), 0,
				  BUTTON_POPUP_CHIPCARD_READID);
	chipcardPopup->insertSeparator();
	chipcardPopup->insertItem(QIconSet(icons.loadIcon("2rightarrow", KIcon::Small)),
				  i18n("&Make card backup-image"), this,
				  SLOT(makeCardBackup_slot()), 0,
				  BUTTON_POPUP_CHIPCARD_SAVEBACKUP);
	chipcardPopup->insertItem(QIconSet(icons.loadIcon("2leftarrow", KIcon::Small)),
				  i18n("&Replay card backup-image"), this,
				  SLOT(replayCardBackup_slot()), 0,
				  BUTTON_POPUP_CHIPCARD_REPLAYBACKUP);
#else // CONFIG_KEYCARD
	chipcardPopup->insertItem(QIconSet(icons.loadIcon("error", KIcon::Small)),
				  i18n("&Feature not configured..."), this,
				  SLOT(noCard_slot()), 0,
				  BUTTON_POPUP_CHIPCARD_NO);
#endif // CONFIG_KEYCARD
	menuBar()->insertItem(i18n("&Chipcard manager"), chipcardPopup);
// "view" popup menu
	viewPopup->insertItem(QIconSet(icons.loadIcon("find", KIcon::Small)),
				i18n("&Find"), this,
				SLOT(find_slot()), 0, BUTTON_POPUP_VIEW_FIND);
	viewPopup->insertSeparator();
	viewPopup->insertItem(QIconSet(icons.loadIcon("halfencrypted", KIcon::Small)),
				i18n("&Lock all entries"), this,
				SLOT(lockWnd_slot()), 0,
				BUTTON_POPUP_VIEW_LOCK);
	viewPopup->insertItem(QIconSet(icons.loadIcon("encrypted", KIcon::Small)),
				i18n("&Deep-lock all entries"), this,
				SLOT(deepLockWnd_slot()), 0,
				BUTTON_POPUP_VIEW_DEEPLOCK);
	viewPopup->insertItem(QIconSet(icons.loadIcon("decrypted", KIcon::Small)),
				i18n("&Unlock all entries"), this,
				SLOT(unlockWnd_slot()), 0,
				BUTTON_POPUP_VIEW_UNLOCK);
	menuBar()->insertItem(i18n("&View"), viewPopup);
// "window" popup menu
	menuBar()->insertItem(i18n("&Window"), windowPopup);
// "options" popup menu
	optionsPopup->insertItem(QIconSet(icons.loadIcon("configure", KIcon::Small)),
				 i18n("&Configure..."), this,
				 SLOT(config_slot()),
				 BUTTON_POPUP_OPTIONS_CONFIG);
	menuBar()->insertItem(i18n("&Options"), optionsPopup);
// "help" popup menu
	helpPopup = helpMenu(QString::null, false);
	menuBar()->insertItem(i18n("&Help"), helpPopup);
}

void PwM::initToolbar()
{
	KIconLoader icons;

	toolBar()->insertButton(icons.loadIcon("filenew", KIcon::Toolbar),
				BUTTON_TOOL_NEW, SIGNAL(clicked(int)), this,
				SLOT(new_slot()), true, i18n("New"));
	toolBar()->insertButton(icons.loadIcon("fileopen", KIcon::Toolbar),
				BUTTON_TOOL_OPEN, SIGNAL(clicked(int)), this,
				SLOT(open_slot()), true, i18n("Open"));
	toolBar()->insertSeparator();
	toolBar()->insertButton(icons.loadIcon("filesave", KIcon::Toolbar),
				BUTTON_TOOL_SAVE, SIGNAL(clicked(int)), this,
				SLOT(save_slot()), true, i18n("Save"));
	toolBar()->insertButton(icons.loadIcon("filesaveas", KIcon::Toolbar),
				BUTTON_TOOL_SAVEAS, SIGNAL(clicked(int)), this,
				SLOT(saveAs_slot()), true, i18n("Save as"));
	toolBar()->insertButton(icons.loadIcon("fileprint", KIcon::Toolbar),
				BUTTON_TOOL_PRINT, SIGNAL(clicked(int)), this,
				SLOT(print_slot()), true, i18n("Print..."));
	toolBar()->insertSeparator();
	toolBar()->insertButton(icons.loadIcon("pencil", KIcon::Toolbar),
				BUTTON_TOOL_ADD, SIGNAL(clicked(int)), this,
				SLOT(addPwd_slot()), true,
				i18n("Add password"));
	toolBar()->insertButton(icons.loadIcon("edit", KIcon::Toolbar),
				BUTTON_TOOL_EDIT, SIGNAL(clicked(int)), this,
				SLOT(editPwd_slot()), true,
				i18n("Edit password"));
	toolBar()->insertButton(icons.loadIcon("editdelete", KIcon::Toolbar),
				BUTTON_TOOL_DEL, SIGNAL(clicked(int)), this,
				SLOT(deletePwd_slot()), true,
				i18n("Delete password"));
	toolBar()->insertSeparator();
	toolBar()->insertButton(icons.loadIcon("find", KIcon::Toolbar),
				BUTTON_TOOL_FIND, SIGNAL(clicked(int)), this,
				SLOT(find_slot()), true, i18n("Find entry"));
	toolBar()->insertSeparator();
	toolBar()->insertButton(icons.loadIcon("halfencrypted", KIcon::Toolbar),
				BUTTON_TOOL_LOCK, SIGNAL(clicked(int)), this,
				SLOT(lockWnd_slot()), true,
				i18n("Lock all entries"));
	toolBar()->insertButton(icons.loadIcon("encrypted", KIcon::Toolbar),
				BUTTON_TOOL_DEEPLOCK, SIGNAL(clicked(int)), this,
				SLOT(deepLockWnd_slot()), true,
				i18n("Deep-Lock all entries"));
	toolBar()->insertButton(icons.loadIcon("decrypted", KIcon::Toolbar),
				BUTTON_TOOL_UNLOCK, SIGNAL(clicked(int)), this,
				SLOT(unlockWnd_slot()), true,
				i18n("Unlock all entries"));
	toolBar()->insertSeparator();
	toolBar()->insertButton(icons.loadIcon("pwg", KIcon::Small),
				BUTTON_TOOL_PWG, SIGNAL(clicked(int)), this,
				SLOT(startPwG_slot()), true, i18n("Start PwG"));
}

void PwM::initDCOP()
{
	if (!conf()->confGlobEnableDcop()) {
		delete_ifnot_null(dcopIf);
		pwmDCOPinterface = 0;
		return;
	}
	dcopIf = new PwManagerDCOPif(PWM_DEFAULT_DCOPOBJ,
				     this, this);
	pwmDCOPinterface = dcopIf;
}

#ifdef CONFIG_KWALLETIF
void PwM::initKWalletEmu()
{
	if (!conf()->confGlobKwalletEmu()) {
		delete_ifnot_null(kwalletEmu);
		importPopup->changeItem(BUTTON_POPUP_IMPORT_KWALLET,
					i18n("&KWallet..."));
		importPopup->setItemEnabled(BUTTON_POPUP_IMPORT_KWALLET, true);
		exportPopup->changeItem(BUTTON_POPUP_EXPORT_KWALLET,
					i18n("&KWallet..."));
		exportPopup->setItemEnabled(BUTTON_POPUP_EXPORT_KWALLET, true);
		return;
	}
	try {
		delete_ifnot_null(kwalletEmu);
		kwalletEmu = new KWalletEmu(PWM_KWALLETEMU_DCOPOBJ,
					    this, this);
	} catch (PwMException e) {
		string errMsg("initializing KWallet emulation failed.  ID: ");
		errMsg += tostr(static_cast<int>(e.getId()));
		errMsg += "  err-message: ";
		errMsg += e.getMessage();
		printWarn(errMsg);
		delete_ifnot_null(kwalletEmu);
		return;
	}
	importPopup->changeItem(BUTTON_POPUP_IMPORT_KWALLET,
				i18n("&KWallet... (not available with KWallet emulation enabled)"));
	importPopup->setItemEnabled(BUTTON_POPUP_IMPORT_KWALLET, false);
	exportPopup->changeItem(BUTTON_POPUP_EXPORT_KWALLET,
				i18n("&KWallet... (not available with KWallet emulation enabled)"));
	exportPopup->setItemEnabled(BUTTON_POPUP_EXPORT_KWALLET, false);
}
#endif // CONFIG_KWALLETIF

void PwM::initMetrics()
{
	QSize s = conf()->confWndMainWndSize();
	if (s.isValid())
		resize(s);
}

void PwM::initBackgroundPix()
{
	delete_ifnot_null(currBg);
	currBg = new QPixmap;
	QString pixPath = conf()->confGlobBgPix();
	currBg->load(pixPath);
	workspace->setPaletteBackgroundPixmap(*currBg);
}

void PwM::showTray()
{
	if (isTrayVisible())
		return;
	tray = new PwMTray(this);
	KIconLoader icons;
	tray->setPixmap(icons.loadIcon(PACKAGE, KIcon::Small));
	tray->show();

	const vector<PwMDocList::listItem> *docList
			= PwMDoc::getOpenDocList()->getList();
	vector<PwMDocList::listItem>::const_iterator i = docList->begin(),
						     end = docList->end();
	while (i != end) {
		connectTray((*i).doc);
		++i;
	}
}

void PwM::connectTray(PwMDoc *doc)
{
	connect(doc, SIGNAL(dataChanged(PwMDoc *)),
		tray, SLOT(updateTree(PwMDoc *)));
	connect(doc, SIGNAL(docClosed(PwMDoc *)),
		tray, SLOT(closeTreeEntry(PwMDoc *)));
	doc->_dataChanged();
}

void PwM::hideTray()
{
	if (!isTrayVisible())
		return;
	tray->hide();
	delete_and_null(tray);
}

void PwM::wndActivated_slot(QWidget *w)
{
	currentView = static_cast<PwMView *>(w);
	enableMdiUi(currentView);
	if (currentView)
		managePopup->setItemEnabled(BUTTON_POPUP_MANAGE_CHANGEMP,
					    curDoc()->getCurrentPw() == "" ?
					     false : true);
	updateWindowPopup();
}

void PwM::hideEvent(QHideEvent *)
{
	if (isMinimized()) {
		if (isTrayVisible())
			hide();

		int mmlock = conf()->confGlobMinimizeLock();
		switch (mmlock) {
		case 0:		// don't lock anything
			break;
		case 1: {	// normal lock
			const vector<PwMDocList::listItem> *docList
				= PwMDoc::getOpenDocList()->getList();
			vector<PwMDocList::listItem>::const_iterator i = docList->begin(),
								     end = docList->end();
			while (i != end) {
				(*i).doc->lockAll(true);
				++i;
			}
			break;
		} case 2: {	// deep-lock
			const vector<PwMDocList::listItem> *docList
					= PwMDoc::getOpenDocList()->getList();
			vector<PwMDocList::listItem>::const_iterator i = docList->begin(),
								     end = docList->end();
			while (i != end) {
				(*i).doc->deepLock();
				++i;
			}
			break;
		} default:
			WARN();
		}
	}
}

void PwM::enableMdiUi(bool enable)
{
	filePopup->setItemEnabled(BUTTON_POPUP_FILE_CLOSE, enable);
	filePopup->setItemEnabled(BUTTON_POPUP_FILE_SAVE, enable);
	filePopup->setItemEnabled(BUTTON_POPUP_FILE_SAVEAS, enable);
	filePopup->setItemEnabled(BUTTON_POPUP_FILE_EXPORT, enable);
	filePopup->setItemEnabled(BUTTON_POPUP_FILE_PRINT, enable);
	managePopup->setItemEnabled(BUTTON_POPUP_MANAGE_ADD, enable);
	managePopup->setItemEnabled(BUTTON_POPUP_MANAGE_EDIT, enable);
	managePopup->setItemEnabled(BUTTON_POPUP_MANAGE_DEL, enable);
	managePopup->setItemEnabled(BUTTON_POPUP_MANAGE_CHANGEMP, enable);
	viewPopup->setItemEnabled(BUTTON_POPUP_VIEW_LOCK, enable);
	viewPopup->setItemEnabled(BUTTON_POPUP_VIEW_DEEPLOCK, enable);
	viewPopup->setItemEnabled(BUTTON_POPUP_VIEW_UNLOCK, enable);
	viewPopup->setItemEnabled(BUTTON_POPUP_VIEW_FIND, enable);
	toolBar()->setItemEnabled(BUTTON_TOOL_SAVE, enable);
	toolBar()->setItemEnabled(BUTTON_TOOL_SAVEAS, enable);
	toolBar()->setItemEnabled(BUTTON_TOOL_PRINT, enable);
	toolBar()->setItemEnabled(BUTTON_TOOL_ADD, enable);
	toolBar()->setItemEnabled(BUTTON_TOOL_EDIT, enable);
	toolBar()->setItemEnabled(BUTTON_TOOL_DEL, enable);
	toolBar()->setItemEnabled(BUTTON_TOOL_LOCK, enable);
	toolBar()->setItemEnabled(BUTTON_TOOL_DEEPLOCK, enable);
	toolBar()->setItemEnabled(BUTTON_TOOL_UNLOCK, enable);
	toolBar()->setItemEnabled(BUTTON_TOOL_FIND, enable);
}

void PwM::new_slot()
{
	makeNewListView();
}

void PwM::open_slot(QString fn)
{
	openDoc(fn);
}

PwMDoc * PwM::openDoc(QString filename, bool openDeepLocked)
{
	makeNewListView();
	PWM_ASSERT(curDoc());
	if (filename == "")
		filename = KFileDialog::getOpenFileName(QString::null,
							i18n("*.pwm|PwM Password file\n"
							     "*|All files"), this);
	if (filename == "")
		goto cancelOpen;
	PwMerror ret;
	while (true) {
		int lockStat = -1;
		if (openDeepLocked) {
			lockStat = 2;
		} else {
			if (conf()->confGlobUnlockOnOpen()) {
				lockStat = 0;
			} else {
				lockStat = 1;
			}
		}
		ret = curDoc()->openDoc(&filename, lockStat);
		if (ret != e_success) {
			if (ret == e_readFile || ret == e_openFile) {
				KMessageBox::error(this,
						   i18n("Could not read file!")
						   + "\n"
						   + filename,
						   i18n("file error"));
				goto cancelOpen;
			}
			if (ret == e_alreadyOpen) {
				KMessageBox::error(this,
						   i18n("This file is already open."),
						   i18n("already open"));
				goto cancelOpen;
			}
			if (ret == e_fileVer) {
				KMessageBox::error(this,
						   i18n
						   ("File-version is not supported!\n"
						    "Did you create this file with an older or newer version of PwM?"),
						   i18n
						   ("incompatible version"));
				goto cancelOpen;
			}
			if (ret == e_wrongPw) {
				continue;
			}
			if (ret == e_noPw) {
				goto cancelOpen;
			}
			if (ret == e_fileFormat) {
				KMessageBox::error(this,
						   i18n
						   ("Sorry, this file has not been recognized "
						    "as a PwM Password file.\n"
						    "Probably you have selected the wrong file."),
						   i18n
						   ("no PwM password-file"));
				goto cancelOpen;
			}
			if (ret == e_fileCorrupt) {
				KMessageBox::error(this,
						   i18n
						   ("File corrupt!\n"
						    "Maybe the media, you stored this file on, "
						    "had bad sectors?"),
						   i18n
						   ("checksum error"));
				goto cancelOpen;
			}
		}
		break;
	}
	showStatMsg(i18n("Successfully opened file."));
	activateMpButton(true);
	updateWindowPopup();
	return curDoc();

      cancelOpen:
	PwMView *v = curView();
	currentView = 0;
	v->hide();
	delete v;
	return 0;
}

PwMView * PwM::makeNewListView()
{
	PwMDoc *doc = makeNewDocument();
	if (isTrayVisible())
		connectTray(doc);
	PwMView *ret = new PwMView(this, workspace, doc);
	ret->document()->setMpwTimeout(conf()->confGlobPwTimeout());
	ret->setFont(conf()->confGlobEntryFont());
	ret->show();
	if (!isVisible())
		currentView = ret;
	updateWindowPopup();
	connect(ret, SIGNAL(closed(PwMView *)),
		this, SLOT(viewClosed(PwMView *)));
	vl.add(ret);
	return ret;
}

PwMDoc * PwM::makeNewDocument()
{
	PwMDoc *doc = new PwMDoc(this);
#ifdef CONFIG_KEYCARD
	doc->setPwMKeyCard(keyCard);
#endif
#ifdef CONFIG_KWALLETIF
	if (kwalletEmu)
		kwalletEmu->connectDocSignals(doc);
#endif
	connect(doc, SIGNAL(dataChanged(PwMDoc *)),
		this, SLOT(updateWindowPopup()));
	return doc;
}

void PwM::updateWindowPopup()
{
#if QT_VERSION >= QT_MAKE_VERSION(3, 2, 0)
	int id = 0;
	QWidgetList widgetList(getMdiWidgetList());
	QWidgetList::iterator i = widgetList.begin(),
			      end = widgetList.end();
	windowPopup->clear();
	while (i != end) {
		windowPopup->insertItem((*i)->caption(), id);
		windowPopup->setItemParameter(id, id);
		windowPopup->connectItem(id, this, SLOT(selectWindowPressed(int)));
		++i;
		++id;
	}
#else // QT_VERSION >= QT_MAKE_VERSION(3, 2, 0)

	int s, i;
	windowPopup->clear();
	QWidgetList widgetList(getMdiWidgetList());
	s = widgetList.count();
	for (i = 0; i < s; ++i) {
		windowPopup->insertItem(widgetList.at(i)->caption(), i);
		windowPopup->setItemParameter(i, i);
		windowPopup->connectItem(i, this, SLOT(selectWindowPressed(int)));
	}
#endif // QT_VERSION >= QT_MAKE_VERSION(3, 2, 0)
}

void PwM::selectWindowPressed(int id)
{
	QWidgetList widgetList(getMdiWidgetList());
	QWidget *w = widgetList.at(id);
	w->setFocus();
}

void PwM::close_slot()
{
	if (curView())
		curView()->close();
}

void PwM::save_slot()
{
	PWM_ASSERT(curDoc());
	if (curDoc()->isDocEmpty()) {
		KMessageBox::information(this,
					 i18n
					 ("Sorry, there's nothing to save.\n"
					  "Please first add some passwords."),
					 i18n("nothing to do"));
		return;
	}
	PwMerror ret = curDoc()->saveDoc(conf()->confGlobCompression());
	if (ret == e_filename) {
		saveAs_slot();
		return;
	} else if (ret == e_weakPw) {
		KMessageBox::error(this,
				   i18n("Error: This is a weak password.\n"
					"Please select another password."),
				   i18n("weak password"));
		return;
	} else if (ret == e_fileBackup) {
		KMessageBox::error(this,
				   i18n("Error: Couldn't make backup-file!"),
				   i18n("backup failed"));
		return;
	} else if (ret != e_success) {
		KMessageBox::error(this,
				   i18n("Error: Couldn't write to file.\n"
					"Please check if you have permission to "
					"write to the file in that directory."),
				   i18n("error while writing"));
		return;
	} else
		showStatMsg(i18n("Successfully saved data."));
}

void PwM::saveAs_slot()
{
	PWM_ASSERT(curDoc());
	if (curDoc()->isDocEmpty()) {
		KMessageBox::information(this,
					 i18n
					 ("Sorry, there's nothing to save.\n"
					  "Please first add some passwords."),
					 i18n("nothing to do"));
		return;
	}
	QString fn(KFileDialog::getSaveFileName(QString::null,
						i18n("*.pwm|PwM Password file"),
						this));
	if (fn == "")
		return;
	if (fn.right(4) != ".pwm")
		fn += ".pwm";

	PwMerror ret = curDoc()->saveDoc(conf()->confGlobCompression(), &fn);
	if (ret != e_success) {
		KMessageBox::error(this,
				   i18n("Error: Couldn't write to file.\n"
					"Please check if you have permission to "
					"write to the file in that directory."),
				   i18n("error while writing"));
		return;
	} else
		showStatMsg(i18n("Successfully saved data."));
}

void PwM::addPwd_slot(QString *pw, PwMDoc *_doc)
{
	PwMDoc *doc;
	if (_doc) {
		doc = _doc;
	} else {
		doc = curDoc();
	}
	PWM_ASSERT(doc);
	AddEntryWndImpl w;
	vector<string> catList;
	doc->getCategoryList(&catList);
	unsigned i, size = catList.size();
	for (i = 0; i < size; ++i) {
		w.addCategory(catList[i].c_str());
	}
	w.setCurrCategory(curView()->getCurrentCategory());
	if (pw)
		w.pwLineEdit->setText(*pw);

      tryAgain:
	if (w.exec() == 1) {
		PwMDataItem d;
		d.desc = w.getDescription().latin1();
		d.name = w.getUsername().latin1();
		d.pw = w.getPassword().latin1();
		d.comment = w.getComment().latin1();
		d.url = w.getUrl().latin1();
		d.launcher = w.getLauncher().latin1();
		PwMerror ret = doc->addEntry(w.getCategory(), &d);
		if (ret == e_entryExists) {
			KMessageBox::error(this,
					   i18n
					   ("Sorry, an entry with this \"Description\", "
					    "\"Username\" and \"Comment\" already exists.\n"
					    "Please set one of these fields to another value."),
					   i18n("entry already exists."));
			goto tryAgain;
		} else if (ret == e_maxAllowedEntr) {
			KMessageBox::error(this, i18n("The maximum possible number of entries "
						      "has been reached. You can't add more entries."),
						 i18n("maximum number of entries"));
			return;
		}
	}
}

void PwM::editPwd_slot(const QString *category, const int *index,
		       PwMDoc *_doc)
{
	PwMDoc *doc;
	if (_doc) {
		doc = _doc;
	} else {
		doc = curDoc();
	}
	PWM_ASSERT(doc);
	if (doc->isDocEmpty())
		return;
	if (doc->isDeepLocked())
		return;
	unsigned int curEntryIndex;
	if (index) {
		curEntryIndex = *index;
	} else {
		if (!(curView()->getCurEntryIndex(&curEntryIndex))) {
			printDebug("couldn't get index");
			return;
		}
	}
	QString curCategory;
	if (category) {
		curCategory = *category;
	} else {
		curCategory = curView()->getCurrentCategory();
	}
	PwMDataItem currItem;
	if (!doc->getEntry(curCategory, curEntryIndex, &currItem, true)) {
		return;
	}

	AddEntryWndImpl w;
	vector<string> catList;
	doc->getCategoryList(&catList);
	unsigned i, size = catList.size();
	for (i = 0; i < size; ++i) {
		w.addCategory(catList[i].c_str());
	}
	w.setCurrCategory(curCategory);
	w.setDescription(currItem.desc.c_str());
	w.setUsername(currItem.name.c_str());
	w.setPassword(currItem.pw.c_str());
	w.setUrl(currItem.url.c_str());
	w.setLauncher(currItem.launcher.c_str());
	w.setComment(currItem.comment.c_str());
	if (w.exec() == 1) {
		currItem.desc = w.getDescription().latin1();
		currItem.name = w.getUsername().latin1();
		currItem.pw = w.getPassword().latin1();
		currItem.comment = w.getComment().latin1();
		currItem.url = w.getUrl().latin1();
		currItem.launcher = w.getLauncher().latin1();
		if (!doc->editEntry(curCategory, w.getCategory(),
					 curEntryIndex, &currItem)) {
			KMessageBox::error(this,
					   i18n("Couldn't edit the entry.\n"
						"Maybe you changed the category and "
						"this entry is already present in the new "
						"category?"),
					   i18n("couldn't edit entry."));
			return;
		}
	}
}

void PwM::deletePwd_slot()
{
	PWM_ASSERT(curDoc());
	if (curDoc()->isDocEmpty())
		return;
	if (curDoc()->isDeepLocked())
		return;
	unsigned int curEntryIndex = 0;
	if (!(curView()->getCurEntryIndex(&curEntryIndex))) {
		printDebug("couldn't get index");
		return;
	}

	PwMDataItem currItem;
	QString curCategory = curView()->getCurrentCategory();
	if (!curDoc()->getEntry(curCategory, curEntryIndex, &currItem)) {
		printDebug("couldn't get entry");
		return;
	}
	if (KMessageBox::
	    questionYesNo(this,
			  i18n
			  ("Do you really want to delete the selected entry") +
			  " \"" + QString(currItem.desc.c_str())
			  + "\" ?", i18n("delete?"))
	    == KMessageBox::Yes) {

		curDoc()->delEntry(curCategory, curEntryIndex);
	}
}

void PwM::changeMasterPwd_slot()
{
	PWM_ASSERT(curDoc());
	curDoc()->changeCurrentPw();
}

void PwM::lockWnd_slot()
{
	PWM_ASSERT(curDoc());
	curDoc()->lockAll(true);
}

void PwM::lockAllDoc_slot()
{
	const vector<PwMDocList::listItem> *docList
			= PwMDoc::getOpenDocList()->getList();
	vector<PwMDocList::listItem>::const_iterator i = docList->begin(),
						     end = docList->end();
	while (i != end) {
		(*i).doc->lockAll(true);
		++i;
	}
}

void PwM::deepLockWnd_slot()
{
	PWM_ASSERT(curDoc());
	curDoc()->deepLock();
}

void PwM::deepLockAllDoc_slot()
{
	const vector<PwMDocList::listItem> *docList
			= PwMDoc::getOpenDocList()->getList();
	vector<PwMDocList::listItem>::const_iterator i = docList->begin(),
						     end = docList->end();
	while (i != end) {
		(*i).doc->deepLock();
		++i;
	}
}

void PwM::unlockWnd_slot()
{
	PWM_ASSERT(curDoc());
	curDoc()->lockAll(false);
}

void PwM::config_slot()
{
	Configuration *conf = Configuration::obj();
	int oldStyle = conf->confWndMainViewStyle();

	// display the configuration window (modal mode)
	if (!conf->showConfWnd(this))
		return;

	int newStyle = conf->confWndMainViewStyle();
	// show/hide tray
	if (conf->confGlobTray())
		showTray();
	else
		hideTray();
	vector<PwMView *>::const_iterator i = vl.getList()->begin(),
					  end = vl.getList()->end();
	PwMView *v;
	while (i != end) {
		v = *i;
		// unlock-without-mpw-timeout
		v->document()->setMpwTimeout(conf->confGlobPwTimeout(), true);
		// auto-lock-timeout
		v->document()->setAutoLockTimeout(conf->confGlobLockTimeout());
		// font
		v->setFont(conf->confGlobEntryFont());
		// window-style
		if (oldStyle != newStyle)
			v->initStyle(newStyle);
		++i;
	}

	initBackgroundPix();
	initDCOP();
	initKWalletEmu();
}

void PwM::trayUndockButton_slot()
{
	if (!isVisible())
		show();
	hideTray();
	conf()->confGlobTray(false);
}

void PwM::activateMpButton(bool activate)
{
	managePopup->setItemEnabled(BUTTON_POPUP_MANAGE_CHANGEMP, activate);
}

void PwM::closeEvent(QCloseEvent *e)
{
	dcopClient->suspend();
	dcopClient->setAcceptCalls(false);
	vector<PwMView *> openViews(*(vl.getList()));
	vector<PwMView *>::iterator i = openViews.begin(),
				    end = openViews.end();
	while (i != end) {
		if (!(*i)->close()) {
			e->ignore();
			dcopClient->setAcceptCalls(true);
			dcopClient->resume();
			return;
		}
		++i;
	}
	e->accept();
}

void PwM::viewClosed(PwMView *view)
{
	vl.del(view);
}

void PwM::find_slot()
{
	PWM_ASSERT(curDoc());
	if (curDoc()->isDocEmpty())
		return;
	if (curDoc()->isDeepLocked())
		return;
	FindWndImpl findWnd(curView());
	findWnd.exec();
}

void PwM::startPwG_slot()
{
	if (!pwgproc.start(KProcess::DontCare)) {
		KMessageBox::error(this,
				   i18n
				   ("Coudn't start the PassWordGenerator \"PwG\"!\n"
				    "Please check if you have installed PwG correctly.\n"
				    "PwG is available from this webpage: "
				    "http://pwg.sourceforge.net"),
				   i18n("PwG failed."));
	}
}

void PwM::exportToText()
{
	PWM_ASSERT(curDoc());
	if (curDoc()->isDocEmpty()) {
		KMessageBox::information(this,
					 i18n
					 ("Sorry, there's nothing to export.\n"
					  "Please first add some passwords."),
					 i18n("nothing to do"));
		return;
	}
	QString fn(KFileDialog::getSaveFileName(QString::null,
						i18n("*|plain-text file"),
						this));
	if (fn == "")
		return;

	PwMerror ret = curDoc()->exportToText(&fn);
	if (ret != e_success) {
		KMessageBox::error(this,
				   i18n("Error: Couldn't write to file.\n"
					"Please check if you have permission to write "
					"to the file in that directory."),
				   i18n("error while writing"));
	} else
		showStatMsg(i18n("Successfully exported data."));
}

void PwM::importFromText()
{
	int ret2;
	bool openedNew = false;
	if (!curView()) {
		makeNewListView();
		openedNew = true;
	} else {
		ret2 = KMessageBox::questionYesNo(this,
						  i18n("Do you want to import the data "
						       "into the current document? (If you "
						       "select \"no\", a new document will be "
						       "opened.)"),
						  i18n("import into this document?"));
		if (ret2 == KMessageBox::No) {
			makeNewListView();
			openedNew = true;
		}
	}
	PWM_ASSERT(curDoc());
	PwMerror ret;
	QString path(KFileDialog::getOpenFileName(QString::null,
						  i18n("*|PWM-exported text file"),
						  this));
	if (path == "")
		goto cancelImport;

	ret = curDoc()->importFromText(&path, 0);
	if (ret == e_fileFormat) {
		KMessageBox::error(this,
				   i18n("Could not read file-format.\n"
					"This seems to be _not_ a valid file "
					"exported by PwM."),
				   i18n("invalid file-format"));
		goto cancelImport;
	} else if (ret == e_invalidArg) {
		BUG();
		goto cancelImport;
	} else if (ret != e_success) {
		KMessageBox::error(this,
				   i18n("Could not import file!\n"
					"Do you have permission to read this file? "
					"Do you have enough free memory?"),
				   i18n("import failed"));
		goto cancelImport;
	}

	return;

      cancelImport:
	if (openedNew) {
		PwMView *v = curView();
		v->hide();
		delete v;
	}
}

void PwM::exportToGpasman()
{
	PWM_ASSERT(curDoc());
	if (curDoc()->isDocEmpty()) {
		KMessageBox::information(this,
					 i18n
					 ("Sorry, there's nothing to export.\n"
					  "Please first add some passwords."),
					 i18n("nothing to do"));
		return;
	}
	QString fn(KFileDialog::getSaveFileName(QString::null,
						i18n("*|Gpasman or Kpasman file"),
						this));
	if (fn == "")
		return;

	PwMerror ret = curDoc()->exportToGpasman(&fn);
	if (ret != e_success) {
		if (ret == e_noPw) {
			return;
		}
		KMessageBox::error(this,
				   i18n("Error: Couldn't write to file.\n"
					"Please check if you have permission to write "
					"to the file in that directory."),
				   i18n("error while writing"));
	} else
		showStatMsg(i18n("Successfully exported data."));
}

void PwM::exportToKWallet()
{
#ifdef CONFIG_KWALLETIF
	PWM_ASSERT(curDoc());
	if (curDoc()->isDocEmpty()) {
		KMessageBox::information(this,
					 i18n
					 ("Sorry, there's nothing to export.\n"
					  "Please first add some passwords."),
					 i18n("nothing to do"));
		return;
	}

	KWalletIf walletIf(this);
	if (walletIf.kwalletExport(curDoc())) {
		KMessageBox::information(this,
					 i18n("Successfully exported the data of the current "
					      "document to KWallet."),
					 i18n("Successfully exported data."));
		showStatMsg(i18n("Successfully exported data."));
	}
#endif // CONFIG_KWALLETIF
}

void PwM::importFromGpasman()
{
	int ret2;
	bool openedNew = false;
	if (!curView()) {
		makeNewListView();
		openedNew = true;
	} else {
		ret2 = KMessageBox::questionYesNo(this,
						  i18n("Do you want to import the data "
						       "into the current document? (If you "
						       "select \"no\", a new document will be "
						       "opened.)"),
						  i18n("import into this document?"));
		if (ret2 == KMessageBox::No) {
			makeNewListView();
			openedNew = true;
		}
	}
	PWM_ASSERT(curDoc());
	PwMerror ret;
	QString path(KFileDialog::getOpenFileName(QString::null,
						  i18n("*|Gpasman or Kpasman file"), this));
	if (path == "")
		goto cancelImport;
	ret = curDoc()->importFromGpasman(&path);
	if (ret == e_wrongPw) {
		ret2 =
		KMessageBox::questionYesNo(this,
				 	   i18n
					   ("This is probably the wrong master-password"
					    "you have typed in.\n"
					    "There is no real way to determine the "
					    "correctness of the password in the Gpasman "
					    "file-format. But I think this "
					    "password ist wrong.\n"
					    "Do you want to continue nevertheless?"),
					   i18n("password error"));
		if (ret2 == KMessageBox::No)
			goto cancelImport;
	} else if (ret != e_success) {
		KMessageBox::error(this,
				   i18n("Could not import file!\n"
					"Do you have permission to read this file?"),
				   i18n("import failed"));
		goto cancelImport;
	}

	return;

      cancelImport:
	if (openedNew) {
		PwMView *v = curView();
		v->hide();
		delete v;
	}
}

void PwM::importKWallet()
{
#ifdef CONFIG_KWALLETIF
	int ret;
	bool openedNew = false;
	if (!curView()) {
		makeNewListView();
		openedNew = true;
	} else {
		ret = KMessageBox::questionYesNo(this,
						 i18n("Do you want to import the data "
						      "into the current document? (If you "
						      "select \"no\", a new document will be "
						      "opened.)"),
						 i18n("import into this document?"));
		if (ret == KMessageBox::No) {
			makeNewListView();
			openedNew = true;
		}
	}
	PWM_ASSERT(curDoc());
	KWalletIf walletIf(this);
	if (walletIf.kwalletImport(curDoc())) {
		KMessageBox::information(this,
					 i18n("Successfully imported the KWallet data "
					      "into the current document."),
					 i18n("successfully imported"));
		showStatMsg(i18n("successfully imported"));
	} else {
		if (openedNew)
			curView()->close();
	}
#endif // CONFIG_KWALLETIF
}

void PwM::print_slot()
{
	if (!curView())
		return;
	PwMPrint p(curDoc(), this);
	p.printNow();
}

void PwM::genNewCard_slot()
{
#ifdef CONFIG_KEYCARD
	keyCard->genNewCard();
#endif
}

void PwM::eraseCard_slot()
{
#ifdef CONFIG_KEYCARD
	keyCard->eraseCard();
#endif
}

void PwM::readCardId_slot()
{
#ifdef CONFIG_KEYCARD
	keyCard->displayKey();
#endif
}

void PwM::makeCardBackup_slot()
{
#ifdef CONFIG_KEYCARD
	keyCard->makeBackupImage();
#endif
}

void PwM::replayCardBackup_slot()
{
#ifdef CONFIG_KEYCARD
	keyCard->replayBackupImage();
#endif
}

void PwM::noCard_slot()
{
#ifndef CONFIG_KEYCARD
	no_keycard_support_msg_box(this);
#endif
}

void PwM::execLauncher_slot()
{
	PWM_ASSERT(curDoc());
	unsigned int curEntryIndex;
	if (!curView()->getCurEntryIndex(&curEntryIndex))
		return;
	bool ret = curDoc()->execLauncher(curView()->getCurrentCategory(),
					  curEntryIndex);
	if (ret)
		showStatMsg(i18n("Executed the \"Launcher\"."));
	else
		showStatMsg(i18n("ERROR: Couldn't execute the \"Launcher\"!"));
}

void PwM::goToURL_slot()
{
	PWM_ASSERT(curDoc());
	unsigned int curEntryIndex;
	if (!curView()->getCurEntryIndex(&curEntryIndex))
		return;
	bool ret = curDoc()->goToURL(curView()->getCurrentCategory(),
				     curEntryIndex);
	if (ret)
		showStatMsg(i18n("started browser with current URL."));
	else
		showStatMsg(i18n("ERROR: Couldn't start browser! Maybe invalid URL?"));
}

void PwM::copyToClipboard(const QString &s)
{
	QClipboard *cb = QApplication::clipboard();
	if (cb->supportsSelection())
		cb->setText(s, QClipboard::Selection);
	cb->setText(s, QClipboard::Clipboard);
}

void PwM::showStatMsg(const QString &msg)
{
	KStatusBar *statBar = statusBar();
	statBar->message(msg, STATUSBAR_MSG_TIMEOUT * 1000);
}

#include "pwm.moc"
