/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004 by Michael Buesch                            *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#ifndef PWMTRAY_H
#define PWMTRAY_H

#include <ksystemtray.h>
#include <kpopupmenu.h>

#include <qpainter.h>

#include <vector>
#include <string>

#include "pwmdoc.h"

using std::vector;
using std::string;

class CustomMenuItemTempl : public QCustomMenuItem
			  , public QObject
{
public:
	CustomMenuItemTempl()
	 : QCustomMenuItem()
	 , QObject()
	{}
	~CustomMenuItemTempl() {}
};

enum ActiveItemExecTask {
	pwToClipboard = 0,
	nameToClipboard,
	descToClipboard,
	urlToClipboard,
	launcherToClipboard,
	commentToClipboard,
	execLauncher,
	goToURL
};

/** implements the "active" part of the tree */
class ActiveTreeItem : public CustomMenuItemTempl
{
	Q_OBJECT
public:
	ActiveTreeItem(const QString &_text, const QFont &_font,
		       ActiveItemExecTask _task, PwMDoc * _doc,
		       int _category, int _entry);
	~ActiveTreeItem();

	void paint(QPainter *p, const QColorGroup & /*cg*/, bool /*act*/,
		   bool /*enabled*/, int x, int y, int w, int h)
	{
		p->setFont(font);
		p->drawText(x, y, w, h, AlignLeft | AlignVCenter |
			    DontClip | ShowPrefix, text);
	}

	QSize sizeHint()
	{
		return QFontMetrics(font).size(AlignLeft | AlignVCenter |
					       ShowPrefix | DontClip,  text);
	}

	bool fullSpan() const
	{
		return false;
	}

	bool isSeparator() const
	{
		return false;
	}

	void setFont(const QFont &f)
	{
		font = f;
	}

public slots:
	/** this is the executive part, that is triggered, when the user clicks it */
	void execIt();

protected:
	QString text;
	QFont font;
	ActiveItemExecTask task;
	PwMDoc *doc;
	int category;
	int entry;
};

/** tray icon implementation */
class PwMTray : public KSystemTray
{
	Q_OBJECT

	struct treeItem
	{
		int menuId;
		PwMDoc *doc;
		KPopupMenu *menu;
		vector<int> activeItems; // ids of all active items
	};

public: 
	PwMTray(QWidget* parent = 0, const char* name = 0);
	~PwMTray();

public slots:
	/** for update-notification from all open documents */
	void updateTree(PwMDoc *document);
	/** when a document has been closed, call this func
	  * to delete its tray-entry
	  */
	void closeTreeEntry(PwMDoc *document);

protected:
	/** holds a list of all open files, its documents and
	  * its menuIDs
	  */
	vector<struct treeItem> tree;

protected:
	/** search if an entry with for the given document
	  * exists and resturn its ID and a pointer to the main
	  * KPopupMenu. If no item is found,
	  * it returns -1 as ID and a NULL-pointer.
	  */
	int findTreeEntry(PwMDoc *doc, KPopupMenu **menu, int *treeItem);
	/** build the main menu-items */
	void buildMain();
	/** rebuilds the tree for the given KPopupMenu entry */
	void rebuildTree(KPopupMenu *popup, PwMDoc *doc,
			 vector<int> *activeItems);
	/** insert a new tree-entry for the given doc and returns the ID.*/
	int insertTreeEntry(PwMDoc *doc, KPopupMenu **popup);
	/** removes a tree entry */
	void removeTreeEntry(int menuId, int treeItem, KPopupMenu **menu);
	/** inserts a new active-tree-item into the given tree entry */
	inline
	void insertActiveTreeItem(KPopupMenu *insertIn, const QString &text,
				  ActiveItemExecTask task, PwMDoc *doc,
				  int docCategory, int docEntry,
				  vector<int> *activeItemsList);
};

#endif
