/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004 by Michael Buesch                            *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "randomizer.h"
#include "pwmexception.h"

#include <stdlib.h>
#include <time.h>


Randomizer * Randomizer::rndObj (0);

Randomizer::Randomizer()
{
	// probe for random devices
	rndDev = fopen("/dev/urandom", "r");
	if (rndDev)
		return;
	rndDev = fopen("/dev/random", "r");
	if (rndDev)
		return;
	// fall back to rand_r()
	seed = time(0);
}

Randomizer::~Randomizer()
{
	if (rndDev) {
		if (fclose(rndDev)) {
			printWarn("failed closing the random-device!");
		}
	}
}

char Randomizer::genRndChar()
{
	if (rndDev) {
		/* we have a rand-device-node */
		return (getc(rndDev));
	} else {
		/* we don't have a random-device-node.
		 * so fall back to rand_r()
		 */
		return (rand_r(&seed) % 0xFF);
	}
}

int Randomizer::genRndInt()
{
	if(rndDev) {
		int ret;
		if (sizeof(int) == 4) {
			(reinterpret_cast<char *>(&ret))[0] = getc(rndDev);
			(reinterpret_cast<char *>(&ret))[1] = getc(rndDev);
			(reinterpret_cast<char *>(&ret))[2] = getc(rndDev);
			(reinterpret_cast<char *>(&ret))[3] = getc(rndDev);
		} else if (sizeof(int) == 8) {
			(reinterpret_cast<char *>(&ret))[0] = getc(rndDev);
			(reinterpret_cast<char *>(&ret))[1] = getc(rndDev);
			(reinterpret_cast<char *>(&ret))[2] = getc(rndDev);
			(reinterpret_cast<char *>(&ret))[3] = getc(rndDev);
			(reinterpret_cast<char *>(&ret))[4] = getc(rndDev);
			(reinterpret_cast<char *>(&ret))[5] = getc(rndDev);
			(reinterpret_cast<char *>(&ret))[6] = getc(rndDev);
			(reinterpret_cast<char *>(&ret))[7] = getc(rndDev);
		} else {
			printWarn(string(__FILE__) + ":" + tostr(__LINE__)
				  + ":  sizeof(int) != 4 && sizeof(int) != 8");
			rndDev = 0;
			seed = time(0);
			return genRndInt();
		}
		return ret;
	} else {
		return rand_r(&seed);
	}
}
