/***************************************************************************
 *                                                                         *
 *   copyright (C) 2004 by Michael Buesch                                  *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "selftest.h"
#include "pwmexception.h"
#include "threadcomm.h"
#include "sha1.h"
#include "blowfish.h"

#include <kmessagebox.h>
#include <iostream>

using std::cerr;
using std::endl;

// quit message sent by thread
#define ST_QUIT_MSG		"done"
// test failed message
#define ST_FAIL_MSG		"failed"
// line end character(s)
#define ST_LINE_END		"\r\n"


static ThreadComm *comm_host;
static const char *failed_was;


SelfTest::SelfTest()
{
	comm_host = new ThreadComm;
	connect(comm_host, SIGNAL(lineAvailable_signal(const string *)),
		this, SLOT(comm_slot(const string *)));
}

SelfTest::~SelfTest()
{
	delete comm_host;
}

void SelfTest::comm_slot(const string *line)
{
	if (line->substr(0, strlen(ST_QUIT_MSG)) == ST_QUIT_MSG) {
		wait();
		delete this;
		return;
	} else if (line->substr(0, strlen(ST_FAIL_MSG)) == ST_FAIL_MSG) {
		wait();
		failed(failed_was);
	}
	BUG();
}

void SelfTest::run()
{
//	printDebug("running self-test...");
	ThreadComm comm_client(comm_host->getSock());

	if (unlikely(!Sha1::selfTest())) {
		failed_was = "SHA1";
		comm_client.send(ST_FAIL_MSG ST_LINE_END);
		return;
	}
	if (unlikely(!Blowfish::selfTest())) {
		failed_was = "BLOWFISH";
		comm_client.send(ST_FAIL_MSG ST_LINE_END);
		return;
	}

	comm_client.send(ST_QUIT_MSG ST_LINE_END);
//	printDebug("self-test done.");
}

void SelfTest::failed(const char *algo)
{
	QString msg("PwManager FATAL ERROR:\n"
		    "Self-test for algorithm \"");
	msg += algo;
	msg += "\" failed!\n\n";
	msg += "It's likely to be either a PwManager BUG or "
	       "bad hardware in your machine. Please contact the "
	       "maintainer of PwManager for this issue.";
	cerr << "\n\n\n" << msg.latin1() << endl;
	KMessageBox::error(0, msg, "PwManager FATAL ERROR");
	::exit(21);
}

#include "selftest.moc"
